<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

require_once dirname(__FILE__) . '/Plugin.php';

/**
 *
 * @author Shopgate GmbH, 35510 Butzbach, DE
 *
 */
class Shopware_Plugins_Backend_SgateShopgatePlugin_Bootstrap extends Shopware_Components_Plugin_Bootstrap
{

// 	############################################################################
// 	## shopware methods
// 	############################################################################
    
    /**
     * (non-PHPdoc)
     *
     * @see Shopware_Components_Plugin_Bootstrap::install()
     */
    public function install()
    {
        $this->uninstall();
        
        $this->eventsInstall();
        
        /*** shopgate setting form install ***/
        $this->formInstall();
        
        /*** shopgate_orders table install ***/
        $this->sqlInstall();
        
        /*** shopgate paymentmeans install ***/
        $this->paymentInstall();
        
        /*** register shop ***/
        $this->registerSystem();
        
        $this->installCache();
        
        return true;
    }
    
    /**
     * (non-PHPdoc)
     *
     * @see Shopware_Components_Plugin_Bootstrap::uninstall()
     */
    public function uninstall()
    {
        $this->unsubscribeCron();
        $this->unsubscribeHooks();
        $this->unsubscribeEvents();
        $this->deleteForm();
        $this->deleteConfig();
        $this->deleteMenuItems();
        
        $this->sqlUninstall();
        $this->paymentUninstall();
        
        return true;
    }
    
    /**
     * (non-PHPdoc)
     *
     * @see Shopware_Components_Plugin_Bootstrap::getInfo()
     */
    public function getInfo()
    {
        return array(
            'version'     => $this->getVersion(),
            'autor'       => 'Shopgate GmbH',
            'copyright'   => 'Copyright @ 2012, Shopgate GmbH',
            'label'       => 'Shopgate - Mobile Shopping',
            'source'      => '',
            'description' => 'Shopgate Schnittstelle - Shopware 3.5',
            'licence'     => 'free',
            'support'     => 'technik@shopgate.com',
            'link'        => 'http://www.shopgate.com',
            'changes'     => '-',
            'revision'    => '-',
        );
    }
    
    /**
     * (non-PHPdoc)
     *
     * @see Shopware_Components_Plugin_Bootstrap::getVersion()
     */
    public function getVersion()
    {
        return "2.3.16";
    }

// 	############################################################################
// 	## shopware methods helper
// 	############################################################################
    
    /**
     *
     */
    public function formInstall()
    {
        /*** plugin config form ***/
        
        $form = $this->Form();
        
        $formDatas = $this->getFormData();
        
        foreach ($formDatas as $i => $formData) {
            $type = $formData["form_type"];
            $key  = $formData["form_key"];
            $data = $formData["form_data"];
            
            $form->setElement($type, $key, $data);
        }
        
        $form->save();
    }
    
    /**
     *
     * Caution with combo-elements!!!:
     * Shopware 3.5.x save the displayField in database instead of valueField!!!
     * http://forum.shopware.de/programmierung-f13/plugconf-comboboxen-speichen-displayfield-statt-valuefield-t5329.html
     *
     * @see http://wiki.shopware.de/Die-Shopware-Plugin-Konfiguration_detail_760_444.html
     * @return array
     */
    protected function getFormData()
    {
        
        $source         = self::getFormDataSources();
        $states         = array();
        $dispatchers    = array();
        $shops          = array();
        $redirect_types = array();
        $server         = array();
        foreach ($source["states"] as $key => $value) {
            $states[] = array($key, $value);
        }
        foreach ($source["dispatchers"] as $key => $value) {
            $dispatchers[] = array($key, $value);
        }
        foreach ($source["shops"] as $key => $value) {
            $shops[] = array($key, $value);
        }
        foreach ($source["redirect_types"] as $key => $value) {
            $redirect_types[] = array($key, $value);
        }
        foreach ($source["servers"] as $key => $value) {
            $server[] = array($key, $value);
        }
        
        $formData = array();
        
        $formData[] = array(
            'form_key'  => 'SGATE_IS_ACTIVE',
            'form_type' => 'checkbox',
            'form_data' => array(
                'label' => 'Aktiv',
                'value' => '0',
                // 						'required' => true,
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_CUSTOMER_NUMBER',
            'form_type' => 'text',
            'form_data' => array(
                'label' => 'Ihre Kundennummer',
                'value' => '',
                // 						'required' => true,
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_SHOP_NUMBER',
            'form_type' => 'text',
            'form_data' => array(
                'label' => 'Ihre Shopnummer',
                'value' => '',
                // 						'required' => true,
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_API_KEY',
            'form_type' => 'text',
            'form_data' => array(
                'label' => 'Ihr API-Key',
                'value' => '',
                // 						'required' => true,
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_ALIAS',
            'form_type' => 'text',
            'form_data' => array(
                'label' => 'Shopgate Alias',
                'value' => '',
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_CNAME',
            'form_type' => 'text',
            'form_data' => array(
                'label' => 'Eingetragener CNAME',
                'value' => '',
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_REDIRECT_TYPE',
            'form_type' => 'combo',
            'form_data' => array(
                'label'       => 'Art der Weiterleitung',
                'description' => 'Verwenden Sie die JavaScript Weiterleitung nur, wenn Sie Cache-Mechniasmen (z.B. Varnish-Cache) einsetzen!',
                'value'       => 'http',
                'attributes'  => array(
                    'valueField'    => 'id',
                    'displayField'  => 'name',
                    'mode'          => 'local',
                    'triggerAction' => 'all',
                    'store'         => 'new Ext.data.ArrayStore({
                                    id: 0,
                                    fields: ["id","name"],
                                    data: ' . json_encode($redirect_types) . '
                                })'
                ),
                'scope'       => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_STATUS_SHIPPING_NOT_BLOCKED',
            'form_type' => 'combo',
            'form_data' => array(
                'label'      => 'Bestellstatus wenn versand nicht blockiert',
                'value'      => 0,
                'attributes' => array(
                    'valueField'    => 'id',
                    'displayField'  => 'name',
                    'mode'          => 'local',
                    'triggerAction' => 'all',
                    'store'         => 'new Ext.data.ArrayStore({
                            id:0,
                            fields:["id","name"],
                            data: ' . json_encode($states) . '
                        })'
                ),
                'scope'      => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_SHIPPING_SERVICE',
            'form_type' => 'combo',
            'form_data' => array(
                'label'      => 'Versandart f&uuml;r Bestellungen',
                'value'      => 9,
                'attributes' => array(
                    'valueField'    => 'id',
                    'displayField'  => 'name',
                    'mode'          => 'local',
                    'triggerAction' => 'all',
                    'store'         => 'new Ext.data.ArrayStore({
                            id:0,
                            fields:["id","name"],
                            data: ' . json_encode($dispatchers) . '
                        })'
                ),
                'scope'      => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_EXPORT_SHIPPING_COSTS', // Premium shipping costs
            'form_type' => 'checkbox',
            'form_data' => array(
                'label' => 'Premium-Versandkosten exportieren',
                'value' => 0,
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_SUBSHOP',
            'form_type' => 'combo',
            'form_data' => array(
                'label'      => 'Sprache',
                'value'      => 1,
                'attributes' => array(
                    'valueField'    => 'id',
                    'displayField'  => 'name',
                    'mode'          => 'local',
                    'triggerAction' => 'all',
                    'store'         => 'new Ext.data.ArrayStore({
                            id:0,
                            fields:["id","name"],
                            data: ' . json_encode($shops) . '
                        })'
                ),
                'scope'      => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_SEND_ORDER_MAIL',
            'form_type' => 'checkbox',
            'form_data' => array(
                'label' => 'Bei neuer Bestellungen eine eMail an den Kunden senden',
                'value' => 0,
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_EAN_ATTRIBUTE_NUMBER',
            'form_type' => 'text',
            'form_data' => array(
                'label' => 'EAN AttributNummer',
                'value' => '',
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_EXPORT_ATTRIBUTE_IDS',
            'form_type' => 'text',
            'form_data' => array(
                'label'       => 'Zu exportierende Attribut-IDs',
                'description' => 'Geben Sie hier mit komma getrennt an, welche Artikelattribute exportiert werden sollen',
                'value'       => '',
                'scope'       => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_SERVER',
            'form_type' => 'combo',
            'form_data' => array(
                'label'       => 'Server',
                'value'       => 'live',
                'description' => 'Dieses Feld nur mit Rücksprache eines Technikers von Shopgate ändern!',
                'attributes'  => array(
                    'valueField'    => 'id',
                    'displayField'  => 'name',
                    'mode'          => 'local',
                    'triggerAction' => 'all',
                    'store'         => 'new Ext.data.ArrayStore({
                            id:0,
                            fields:["id","name"],
                            data: ' . json_encode($server) . '
                        })'
                ),
                'scope'       => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $formData[] = array(
            'form_key'  => 'SGATE_SERVER_URL',
            'form_type' => 'text',
            'form_data' => array(
                'label'       => 'Server-URL',
                'description' => 'Dieses Feld nur mit Rücksprache eines Technikers von Shopgate ändern!',
                'value'       => '',
                'scope'       => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        
        return $formData;
    }
    
    public static function getFormDataSources()
    {
        $qry    =
            Shopware()->Db()->query("SELECT * FROM `s_core_states` s WHERE s.group = 'state' ORDER BY s.position");
        $states = array();
        foreach ($qry->fetchAll() as $state) {
            /* @var $state array(id, description, position, group, mail) */
            
            $states[$state["id"]] = $state["description"];
            
        }
        
        $qry         = Shopware()->Db()->query("SELECT * FROM `s_premium_dispatch` s ORDER BY s.position");
        $dispatchers = array();
        foreach ($qry->fetchAll() as $dispatcher) {
            /* @var $dispatcher array(id, name, ..., position, ...) */
            $dispatchers[$dispatcher["id"]] = $dispatcher["name"];
        }
        
        $qry   = Shopware()->Db()->query("SELECT * FROM s_core_multilanguage");
        $shops = array();
        foreach ($qry->fetchAll() as $shop) {
            /* @var $shop array(id, name, ...) */
            
            $shops[$shop["id"]] = $shop["name"];
        }
        
        $redirect_types = array(
            Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Config::REDIRECT_TYPE_HTTP => 'HTTP (Empfohlen)',
            Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Config::REDIRECT_TYPE_JS   => 'JavaScript'
        );
        
        $servers = array(
            "live"   => "Live",
            "pg"     => "Playground",
            "custom" => "Eigener",
        );
        
        $source = array(
            "states"         => $states,
            "dispatchers"    => $dispatchers,
            "shops"          => $shops,
            "redirect_types" => $redirect_types,
            "servers"        => $servers,
        );
        
        return $source;
    }
    
    private function installCache()
    {
        $cacheDir = Shopware()->DocPath() . DS . 'cache' . DS;
        
        $dirs = array(
            'shopgate' . DS . 'cache',
            'shopgate' . DS . 'export',
            'shopgate' . DS . 'log',
        );
        
        foreach ($dirs as $dir) {
            $path = $cacheDir . $dir;
            @mkdir($path, 0777, true);
        }
    }
    
    /**
     * Creates database table(s)
     *
     * @return
     */
    public function sqlInstall()
    {
        
        // Install Database if not exists
        Shopware()->Db()->query(
            "CREATE TABLE IF NOT EXISTS `s_shopgate_orders` (
            `id` INT NOT NULL AUTO_INCREMENT,
            `orderID` INT NOT NULL,
            `shopgate_order_number` BIGINT NOT NULL,
            `is_shipping_blocked` INT(1) NOT NULL DEFAULT 1,
            `order_import_complete` INT(1) NOT NULL DEFAULT 0,
            `is_sent_to_shopgate` INT(1) NOT NULL DEFAULT 0,
            `is_cancellation_sent_to_shopgate` INT(1) NOT NULL DEFAULT 0,
            `reported_cancellations` TEXT,
            `received_data` TEXT,
            PRIMARY KEY (`id`)
        ) ENGINE = MyISAM AUTO_INCREMENT=1 DEFAULT CHARSET=utf8;"
        );
        
        
        // build list of existinf database fields
        $result         = Shopware()->Db()->fetchAll("SHOW COLUMNS FROM `s_shopgate_orders`");
        $existingFields = array();
        foreach ($result as $row) {
            $existingFields[$row['Field']] = true;
        }
        
        // check if field exists
        if (empty($existingFields['is_shipping_blocked'])) {
            Shopware()->Db()->query(
                "ALTER TABLE `s_shopgate_orders` ADD COLUMN `is_shipping_blocked` INT(1) NOT NULL DEFAULT 1;"
            );
        }
        if (empty($existingFields['is_cancellation_sent_to_shopgate'])) {
            Shopware()->Db()->query(
                "ALTER TABLE `s_shopgate_orders` ADD COLUMN `is_cancellation_sent_to_shopgate` INT(1) NOT NULL DEFAULT 0;"
            );
        }
        if (empty($existingFields['reported_cancellations'])) {
            Shopware()->Db()->query("ALTER TABLE `s_shopgate_orders` ADD COLUMN `reported_cancellations` TEXT;");
        }
        if (empty($existingFields['received_data'])) {
            Shopware()->Db()->query("ALTER TABLE `s_shopgate_orders` ADD COLUMN `received_data` TEXT;");
        }
        
        // Update some fields if they are already there (may change names as well)
        // -> shopgate_order_id => id
        if (!empty($existingFields['shopgate_order_id'])) {
            Shopware()->Db()->query(
                "ALTER TABLE `s_shopgate_orders` CHANGE COLUMN `shopgate_order_id` `id` INT NOT NULL AUTO_INCREMENT;"
            );
            unset($existingFields['shopgate_order_id']);
            $existingFields['id'] = true;
        }
        if (empty($existingFields['id'])) {
            Shopware()->Db()->query(
                "ALTER TABLE `s_shopgate_orders` ADD COLUMN `id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY FIRST;"
            );
        }
        // -> shopgate_order_number => shopgate_order_number
        if (!empty($existingFields['shopgate_order_number'])) {
            Shopware()->Db()->query(
                "ALTER TABLE `s_shopgate_orders` CHANGE COLUMN `shopgate_order_number` `shopgate_order_number` VARCHAR( 20 ) NOT NULL;"
            );
        }
        if (empty($existingFields['shopgate_order_number'])) {
            Shopware()->Db()->query(
                "ALTER TABLE `s_shopgate_orders` ADD COLUMN `shopgate_order_number` VARCHAR( 20 ) NOT NULL;"
            );
        }
        // -> order_import_complete
        if (empty($existingFields['order_import_complete'])) {
            Shopware()->Db()->query(
                "ALTER TABLE `s_shopgate_orders` ADD COLUMN `order_import_complete` INT(1) NOT NULL DEFAULT 0;"
            );
        }
        // -> shipped => is_sent_to_shopgate
        if (!empty($existingFields['shipped'])) {
            Shopware()->Db()->query(
                "ALTER TABLE `s_shopgate_orders` CHANGE COLUMN `shipped` `is_sent_to_shopgate` INT(1) NOT NULL DEFAULT 0;"
            );
            unset($existingFields['shipped']);
            $existingFields['is_sent_to_shopgate'] = true;
        }
        if (empty($existingFields['is_sent_to_shopgate'])) {
            Shopware()->Db()->query(
                "ALTER TABLE `s_shopgate_orders` ADD COLUMN `is_sent_to_shopgate` INT(1) NOT NULL DEFAULT 0;"
            );
        }
        
        // Drop innecessary fields
        if (!empty($existingFields['confirm_link'])) {
            Shopware()->Db()->query("ALTER TABLE `s_shopgate_orders` DROP COLUMN `confirm_link`;");
            unset($existingFields['confirm_link']);
        }
        
        return true;
    }
    
    /**
     * Deletes database table(s)
     *
     * @return
     */
    public function sqlUninstall()
    {
        /*** dont delete table s_shopgate_orders ***/
        // Shopware()->Db()->query("DROP TABLE `s_shopgate_orders`;");
        return true;
    }
    
    /**
     * Installs paymentmean shopgate
     *
     * @return
     */
    public function paymentInstall()
    {
        $aPayments = array(
            array(
                "name"        => "shopgate",
                "description" => "Shopgate Zahlungsart - Der Zahlungsfluss l&auml;uft &uuml;ber das Shopgate-Konto",
                "template"    => "prepayment.tpl",
                "active"      => 0
            ),
            array(
                "name"        => "mobile_payment",
                "description" => "Generische Zahlungsart von Shopgate - Greift, wenn keine Passende Zahlungsart gefunden wurde",
                "template"    => "prepayment.tpl",
                "active"      => 0
            ),
        
        );
        
        foreach ($aPayments as $aPayment) {
            Shopware()->Db()->insert("s_core_paymentmeans", $aPayment);
        }
        
        return true;
    }
    
    /**
     * Uninstalls paymentmean shopgate
     *
     * @return
     */
    public function paymentUninstall()
    {
        Shopware()->Db()->query("DELETE FROM s_core_paymentmeans WHERE name = 'shopgate'");
        Shopware()->Db()->query("DELETE FROM s_core_paymentmeans WHERE name = 'mobile_payment'");
        
        return true;
    }
    
    public function eventsInstall()
    {
        /*** frontend event -> framework-api access ***/
        $event = $this->createEvent(
            'Enlight_Controller_Dispatcher_ControllerPath_Frontend_Shopgate',
            'onGetControllerPathFrontend'
        );
        $this->subscribeEvent($event);
        
        /*** cronjob order confirm shipping and cleanup***/
        $cronjob = $this->createEvent(
            'Shopware_CronJob_CronShopgate',
            'onRun'
        );
        $this->subscribeEvent($cronjob);
        $this->subscribeCron('Shopgate Cron', 'CronShopgate', 1800, true);
        
        /*** mobile redirect ***/
        $event = $this->createEvent(
            'Enlight_Controller_Front_SendResponse',
            'mobileRedirect'
        );
        $this->subscribeEvent($event);
    }

// 	############################################################################
// 	## Events
// 	############################################################################
    
    /**
     * Listener for controller -> call framework
     *
     * @static
     *
     * @param Enlight_Event_EventArgs $args
     *
     * @return
     */
    public static function onGetControllerPathFrontend(Enlight_Event_EventArgs $args)
    {
        define("SHOPGATE_PLUGIN_VERSION", self::getVersion());
        
        // Aktueller Shop
        $shop   = Shopware()->Shop()->getId();
        $locale = Shopware()->Locale()->getId();
        if (empty($shop) || empty($locale)) {
            throw new Enlight_Exception('Plugin Error - Shop-Id or Locale-Id is empty');
        }
        
        return dirname(__FILE__) . '/Controllers/Frontend/shopgate.php';
    }
    
    public static function onGetControllerPathBackend(Enlight_Event_EventArgs $args)
    {
        return dirname(__FILE__) . '/Controllers/Backend/shopgate.php';
    }
    
    /**
     * Cronjob to auto-confirm shipping of shopgate-orders
     */
    public static function onRun(Shopware_Components_Cron_CronJob $args)
    {
        Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Cron::onRun($args);
        
        return 1;
    }
    
    /**
     * Make a Redirect to the Mobile-Page
     *
     * Will called at the Shopware-Event 'Enlight_Controller_Front_SendResponse'
     *
     * @param Enlight_Event_EventArgs $args
     */
    public static function mobileRedirect(Enlight_Event_EventArgs $args)
    {
        $redirector = new Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Redirect($args);
        $redirector->redirect();
        
        return;
    }
    
    /**
     * Register shop at shopgate
     */
    public function registerSystem()
    {
        $installer = new Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Install();
        $installer->updateShopgateInstall();
    }
}
