<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

require_once dirname(__FILE__) . '/../shopgate_library/shopgate.php';

class Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Config extends ShopgateConfig
{
    
    const REDIRECT_TYPE_HTTP = 'http';
    const REDIRECT_TYPE_JS   = 'js';
    
    /**
     *
     * @var bool
     */
    protected $is_active;
    
    protected $sub_shop;
    
    protected $shipping_service;
    
    protected $shipping_not_blocked_status;
    
    protected $send_order_mail;
    
    protected $redirect_type;
    
    protected $export_attribute_ids;
    
    protected $ean_attribute_number;
    
    protected $export_premium_shipping_costs;
    
    public function startup()
    {
        /*** shopinformation / pluginconfiguration ***/
        $pConfig = Shopware()->Plugins()->Backend()->SgateShopgatePlugin()->Config();
        
        $this->setEnablePing(true);
        $this->setEnableAddOrder(true);
        $this->setEnableUpdateOrder(true);
        $this->setEnableGetCustomer(true);
        $this->setEnableGetCategoriesCsv(true);
        $this->setEnableGetItemsCsv(true);
        $this->setEnableGetReviewsCsv(true);
        $this->setEnableCron(true);
        $this->setEnableCheckCart(true);
        $this->setEnableRedeemCoupons(true);
        $this->setEnableGetLogFile(true);
        $this->setEnableClearLogFile(true);
        $this->setEnableClearCache(true);
        $this->setEnableRedirectKeywordUpdate(true);
        $this->setEnableMobileWebsite(true);
        $this->setEnableGetSettings(true);
        $this->setEnableDefaultRedirect(0);
        
        $cacheDir = Shopware()->DocPath() . DS . 'cache' . DS . 'shopgate' . DS;
        
        /*** create config-array ***/
        $shopgate_config = array(
            'apikey'               => $pConfig->SGATE_API_KEY,
            'shop_number'          => $pConfig->SGATE_SHOP_NUMBER,
            'customer_number'      => $pConfig->SGATE_CUSTOMER_NUMBER,
            'server'               => $pConfig->SGATE_SERVER,
            'api_url'              => $pConfig->SGATE_SERVER_URL,
            'plugin_name'          => 'shopware',
            'cname'                => $pConfig->SGATE_CNAME,
            'alias'                => $pConfig->SGATE_ALIAS,
            'is_active'            => $pConfig->SGATE_IS_ACTIVE,
            'redirect_type'        => $pConfig->SGATE_REDIRECT_TYPE,
            'ean_attribute_number' => $pConfig->SGATE_EAN_ATTRIBUTE_NUMBER,
            'send_order_mail'      => $pConfig->SGATE_SEND_ORDER_MAIL,
            'shipping_not_blocked_status'
                                   => $pConfig->SGATE_STATUS_SHIPPING_NOT_BLOCKED,
            'export_premium_shipping_costs'
                                   => $pConfig->SGATE_EXPORT_SHIPPING_COSTS,
            'export_attribute_ids' => $pConfig->SGATE_EXPORT_ATTRIBUTE_IDS,
            'shipping_service'     => $pConfig->SGATE_SHIPPING_SERVICE,
            'cache_folder_path'    => $cacheDir . 'cache' . DS,
            'log_folder_path'      => $cacheDir . 'log' . DS,
            'export_folder_path'   => $cacheDir . 'export' . DS,
        );
        
        /* Convert values from config to it real value
         * Shopware 3.5 returns displayField as valueField...
         * it is a bug in shopware
         *
         * @see http://forum.shopware.de/programmierung-f13/plugconf-comboboxen-speichen-displayfield-statt-valuefield-t5329.html
         */
        $source = Shopware_Plugins_Backend_SgateShopgatePlugin_Bootstrap::getFormDataSources();
        extract($source);
        $shopgate_config["server"] = array_search($shopgate_config["server"], $servers);
        $shopgate_config["shipping_not_blocked_status"] = array_search($shopgate_config["shipping_not_blocked_status"], $states);
        $shopgate_config["shipping_service"] = array_search($shopgate_config["shipping_service"], $dispatchers);
        $shopgate_config["sub_shop"] = array_search($shopgate_config["sub_shop"], $shops);
        $shopgate_config["redirect_type"] = array_search($shopgate_config["redirect_type"], $redirect_types);
        
        $this->loadArray($shopgate_config);
        
        @mkdir($this->getExportFolderPath(), 0777, true);
        @mkdir($this->getLogFolderPath(), 0777, true);
        @mkdir($this->getCacheFolderPath(), 0777, true);
        
        return true;
    }
    
    /**
     *
     * @return bool
     */
    public function getIsActive()
    {
        return $this->is_active;
    }
    
    /**
     *
     * @param bool $value
     */
    public function setIsActive($value)
    {
        $this->is_active = $value;
    }
    
    /**
     * @return int
     */
    public function getShippingService()
    {
        return $this->shipping_service;
    }
    
    public function setShippingService($value)
    {
        $this->shipping_service = $value;
    }
    
    public function getShippingNotBlockedStatus()
    {
        return $this->shipping_not_blocked_status;
    }
    
    public function setShippingNotBlockedStatus($value)
    {
        $this->shipping_not_blocked_status = $value;
    }
    
    /**
     *
     * @return bool
     */
    public function getSendOrderMail()
    {
        return $this->send_order_mail;
    }
    
    /**
     *
     * @param bool $value
     */
    public function setSendOrderMail($value)
    {
        $this->send_order_mail = $value;
    }
    
    /**
     *
     * @return string http|js
     */
    public function getRedirectType()
    {
        return $this->redirect_type;
    }
    
    /**
     *
     * @param string http|js $value
     */
    public function setRedirectType($value)
    {
        $this->redirect_type = $value;
    }
    
    /**
     *
     * @param bool $asArray
     *
     * @return array|string
     */
    public function getExportAttributeIds($asArray = false)
    {
        $value = $this->export_attribute_ids;
        $value = preg_replace("/\s/i", "", $value);
        
        if ($asArray) {
            $value = explode(",", $value);
        }
        
        return $value;
    }
    
    /**
     *
     * @param array|string $value
     */
    public function setExportAttributeIds($value)
    {
        if (is_array($value)) {
            $value = implode(",", $value);
        }
        $this->export_attribute_ids = $value;
    }
    
    public function getEanAttributeNumber()
    {
        return $this->ean_attribute_number;
    }
    
    public function setEanAttributeNumber($value)
    {
        $this->ean_attribute_number = $value;
    }
    
    
    /**
     *
     * @return bool
     */
    public function getExportPremiumShippingCosts()
    {
        return $this->export_premium_shipping_costs;
    }
    
    /**
     *
     * @param bool $value
     */
    public function setExportPremiumShippingCosts($value)
    {
        $this->export_premium_shipping_costs = $value;
    }
}