<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

include_once dirname(__FILE__) . '../shopgate_library/shopgate.php';

class Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Cron
{
    
    public static function onRun(Shopware_Components_Cron_CronJob $args)
    {
        $messageDummy    = "";
        $errorCountDummy = 0;
        
        self::cleanupOrders($messageDummy, $errorCountDummy);
        self::checkOrderStatus($messageDummy, $errorCountDummy);
    }
    
    /**
     * Clean up table s_shopgate_orders
     *
     * Delete Rows with old Orders
     */
    public static function cleanupOrders(&$message, &$errorcount)
    {
        $qry = "DELETE FROM `s_shopgate_orders` WHERE orderID NOT IN (SELECT id FROM `s_order`)";
        Shopware()->Db()->query($qry);
    }
    
    /**
     * Check for orders which are already send and send the order-sate to Shopgate
     *
     */
    public static function checkOrderStatus(&$message, &$errorcount)
    {
        $config      = new Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Config();
        $builder     = new ShopgateBuilder($config);
        $merchantApi = $builder->buildMerchantApi();
        
        $sql = "
            SELECT
                so.shopgate_order_number,
                o.id AS orderID,
                o.status,
                o.cleared
            FROM `s_shopgate_orders` so
            JOIN `s_order` o ON ( o.id = so.orderID )
            WHERE so.is_sent_to_shopgate = 0
        ";
        
        $acceptStates = array(
            ShopgatePluginShopware::STATE_SHIPPING_SHIPPED_COMPLETED,
            ShopgatePluginShopware::STATE_SHIPPING_COMPLETED
        );
        
        $rResult = Shopware()->Db()->query($sql);
        while ($aRow = $rResult->fetch()) {
            if (in_array( $aRow["status"], $acceptStates)) { // Status 'Shipping complete'
                $qry = "
                    UPDATE `s_shopgate_orders`
                    SET is_sent_to_shopgate = 1
                    WHERE shopgate_order_number = {$aRow["shopgate_order_number"]}
                ";
                try {
                    $merchantApi->setOrderShippingCompleted($aRow["shopgate_order_number"]);
                    
                    Shopware()->Db()->query($qry);
                } catch (ShopgateMerchantApiException $e) {
                    if (
                        ($e->getCode() == ShopgateMerchantApiException::ORDER_SHIPPING_STATUS_ALREADY_COMPLETED) ||
                        ($e->getCode() == ShopgateMerchantApiException::ORDER_ALREADY_COMPLETED)
                    ) {
                        Shopware()->Db()->query( $qry );
                    } else {
                        $errorcount++;
                    }
                } catch (Exception $e) {
                    $errorcount++;
                    Shopware()->Log()->err($e->getMessage());
                }
            }
        }
    }
}