<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

class Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Install
{
    
    /**
     * request url
     */
    const URL_TO_UPDATE_SHOPGATE = 'https://api.shopgate.com/log';
    
    /**
     * interface installation action
     */
    const TYPE_INSTALL = "interface_install";
    
    /**
     * internal shopgate system identifier
     */
    const INTERNAL_SHOPSYSTEM_ID = 120;
    
    /**
     * shopware system object
     */
    protected $sSYSTEM;
    
    /**
     * reference date
     */
    protected $date;
    
    /**
     * prepare stuff
     */
    public function __construct()
    {
        $this->sSYSTEM = Shopware()->System();
        $this->date    = date('Y-m-d H:i:s', strtotime(date('Y-m-d H:i:s') . '-1 months'));
    }
    
    /**
     * build registration data and call
     */
    public function updateShopgateInstall($type = self::TYPE_INSTALL)
    {
        
        $subshops = array();
        
        $sql = "
            SELECT store.*, cur.currency
            FROM s_core_multilanguage AS store
            LEFT JOIN s_core_currencies AS cur ON cur.id = store.defaultcurrency
        ";
        
        foreach (Shopware()->Db()->fetchAll($sql) as $subshop) {
            
            $rootCatId = $subshop['parentID'];
            $catIds    = Shopware()->Api()->Export()->sCategories(array(), $rootCatId);
            
            $domains = explode("\n", $subshop['domainaliase']);
            $domain  = trim(reset($domains));
            $url     = (!empty($domain))
                ? $domain
                : $this->sSYSTEM->sPathBase;
            
            $subshops[] = array(
                'uid'              => $subshop['id'],
                'url'              => $url,
                'name'             => $subshop['name'],
                'contact_name'     => '',
                'contact_phone'    => '',
                'contact_email'    => $this->sSYSTEM->sCONFIG['sMAIL'],
                'stats_items'      => $this->getItems($catIds),
                'stats_categories' => count($catIds),
                'stats_orders'     => $this->getOrders($subshop['id']),
                'stats_acs'        => $this->calculateAverage($subshop['id']),
                'stats_currency'   => $subshop['currency'],
            );
        }
        
        $data = array(
            'action'              => $type,
            'uid'                 => $this->getUid(),
            'plugin_version'      => Shopware()->Plugins()->Backend()->SgateShopgatePlugin()->getVersion(),
            'shopping_system_id'  => self::INTERNAL_SHOPSYSTEM_ID,
            'stats_unique_visits' => $this->getVisitors(),
            'stats_mobile_visits' => 0,
            'subshops'            => $subshops
        );
        
        try {
            $client = new Zend_Http_Client(self::URL_TO_UPDATE_SHOPGATE);
            $client->setParameterPost($data);
            $client->request(Zend_Http_Client::POST);
        } catch (Exception $e) {
            $this->log("Shopgate_Framework Message: " . self::URL_TO_UPDATE_SHOPGATE . " could not be reached.");
        }
    }
    
    /**
     * create log file entries
     *
     * @param        $msg
     * @param string $type
     *
     * @return bool
     */
    protected function log($msg, $type = ShopgateLogger::LOGTYPE_ERROR)
    {
        return ShopgateLogger::getInstance()->log($msg, $type);
    }
    
    /**
     * get uid to clarify identification in shopgate system
     *
     * @return string
     */
    protected function getUid()
    {
        $host		= rtrim($this->sSYSTEM->sCONFIG["sHOST"], '/') . '/' . trim($this->sSYSTEM->sCONFIG["sBASEPATH"], '/');
        $accountId	= $this->sSYSTEM->sCONFIG['sACCOUNTID'];

        return sha1(rtrim($host, '/') . $accountId);
    }
    
    /**
     * get product count by categories
     *
     * @param array $categories
     *
     * @return int
     */
    protected function getItems($categories)
    {
        $sql = "
            SELECT count(DISTINCT a.id) FROM s_articles AS a
                INNER JOIN s_articles_categories AS ac ON ac.articleID = a.id
            WHERE ac.categoryID IN (" . implode(',', array_keys($categories)) . ")
        ";
        
        return Shopware()->Db()->fetchOne($sql);
    }
    
    /**
     * get orders count unfiltered
     *
     * @param int $shopId
     *
     * @return int
     */
    protected function getOrders($shopId)
    {
        $sql = "
            SELECT count(id)
            FROM s_order
            WHERE ordertime >= '" . $this->date . "'
            AND subshopID = '" . $shopId . "'
        ";
        
        return Shopware()->Db()->fetchOne($sql);
    }
    
    /**
     * @param int $shopId
     *
     * @return float
     */
    protected function calculateAverage($shopId)
    {
        $sql = "SELECT IFNULL(AVG(invoice_amount), 0) FROM s_order WHERE subshopID = '" . $shopId . "'";
        
        return round(Shopware()->Db()->fetchOne($sql), 2);
    }
    
    /**
     * get visitor data unfiltered
     *
     * @return int
     */
    protected function getVisitors()
    {
        $sql = "SELECT IFNULL(SUM(uniquevisits), 0) FROM s_statistics_visitors WHERE datum >= '" .
            date('Y-m-d', strtotime($this->date)) . "'";
        
        return Shopware()->Db()->fetchOne($sql);
    }
}