<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

class Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Order
{
    
    /**
     *
     * @param string $ordernumber
     */
    public function cancelOrder($ordernumber)
    {
        
        // Fetch shopgate order from database by given shopnumber
        $order = Shopware()->Db()->fetchRow(
            "SELECT * FROM `s_shopgate_orders` WHERE shopgate_order_number = ?",
            array($ordernumber)
        );
        
        // return false if order not found
        if (!$order) {
            return false;
        }
        
        // unserialize shopgate data to get object of a ShopgateOrder instance
        $shopgateOrder = unserialize($order["received_data"]);
        
        // if unserialize failed $shopgateOrder will be null
        // in this case resync the order
        if (!$shopgateOrder) {
            $shopgateOrder = $this->syncOrder($ordernumber);
        }
        
        // load shopware order from database
        $shopwareOrder = Shopware()->Db()->fetchRow(
            "SELECT * FROM `s_order` WHERE id = ?",
            array($order["orderID"])
        );

        $bCancelCompleteOrder = false;
        $aCancellationItems   = array();
        $bCancleShipping      = false;
        
        if ($shopwareOrder["status"] == 4) {
            // if shopware order status has id '4' it is a full cancellation
            
            $bCancelCompleteOrder = true;
            $bCancleShipping      = true;
            
        } else {
            
            // otherwise check for partial cancellation
            
            // build shopgate order item stack to access items by item number
            $itemStack = array();
            foreach ($shopgateOrder->getItems() as $item) {
                /* @var $item \ShopgateOrderItem */
                $key = $item->getItemNumber();
                
                $itemStack[$key] = $item;
            }
            
            $totalCanceld = 0;
            
            $totalOrderQty = Shopware()->Db()->fetchOne(
                "SELECT SUM( quantity ) FROM `s_order_details` WHERE orderID = ? AND articleID != 0",
                array($order["orderID"])
            );
            
            $shopwareOrderItems = Shopware()->Db()->fetchAll(
                "SELECT * FROM `s_order_details` WHERE orderID = ?",
                array($order["orderID"])
            );
        
            // already reported cancellations
            $aAlreadyCanceledItems = unserialize($order["reported_cancellations"]);
            if (!$aAlreadyCanceledItems) {
                $aAlreadyCanceledItems = array();
            }
            
            foreach ($shopwareOrderItems as $shopwareOrderItem) {
                if ($shopwareOrderItem["status"] == 2) {
                    if ($sgOrderItem = $itemStack[$shopwareOrderItem["articleordernumber"]]) {
                        /* @var $sgOrderItem ShopgateOrderItem */
                        $totalCanceld += $shopwareOrderItem["quantity"];
                        
                        if ($aAlreadyCanceledItems[$sgOrderItem->getItemNumber()]) {
                            continue;
                        }
                        
                        $aCancellationItems[$sgOrderItem->getItemNumber()] = array(
                            "item_number" => $sgOrderItem->getItemNumber(),
                            "quantity"    => $sgOrderItem->getQuantity()
                        );
                        
                        $aAlreadyCanceledItems[$sgOrderItem->getItemNumber()] = $sgOrderItem->getQuantity();
                    }
                }
            }
            
            // if all items are cancelled then cancel the whole order
            if ($totalOrderQty <= $totalCanceld) {
                $bCancelCompleteOrder = true;
            }
            
            $order["reported_cancellations"] = serialize($aAlreadyCanceledItems);
            
        }
        
        if ($bCancelCompleteOrder || $aCancellationItems) {
            
            try {
                if ($bCancelCompleteOrder) {
                    $order["is_cancellation_sent_to_shopgate"] = "1";
                }
                
                $sma = $this->getShopgateMerchantApiInstance();
                $sma->cancelOrder(
                    $ordernumber, $bCancelCompleteOrder, $aCancellationItems, $bCancleShipping,
                    "Order was canceld in shopware"
                );
                
            } catch (ShopgateMerchantApiException $e) {
                if ($e->getCode() == 222) { // already canceld
                    $order["is_cancellation_sent_to_shopgate"] = "1";
                    
                    $e = null;
                }
            }
            
        } else {
            return false;
        }
        
        if ($e) {
            throw $e;
        }
        
        Shopware()->Db()->update("s_shopgate_orders", $order, array("shopgate_order_number = '{$ordernumber}'"));
        
        return true;
    }
    
    /**
     *
     * @param string $ordernumber
     *
     * @return ShopgateOrder
     */
    protected function syncOrder($ordernumber)
    {
        $order = Shopware()->Db()->fetchRow(
            "SELECT * FROM `s_shopgate_orders` WHERE shopgate_order_number = ?",
            array($ordernumber)
        );
    
        $oShopgateMerchantApi = $this->getShopgateMerchantApiInstance();
        $shopgateOrders = $oShopgateMerchantApi
                ->getOrders(array("order_numbers" => array($ordernumber), "with_items" => true))
                ->getData();
        $shopgateOrder = array_shift($shopgateOrders);
        
        $order["received_data"] = serialize($shopgateOrder);
        
        Shopware()->Db()->update("s_shopgate_orders", $order, array("shopgate_order_number = '{$ordernumber}'"));
        
        return $shopgateOrder;
    }
    
    protected function getShopgateMerchantApiInstance()
    {
        $config = $this->getConfig();
        
        $builder = new ShopgateBuilder($config);
        $oSMA    = $builder->buildMerchantApi();
        
        return $oSMA;
    }
    
    /**
     *
     * @return Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Config
     */
    protected function getConfig()
    {
        static $config = null;
        
        if (!$config) {
            $config = new Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Config();
        }
        
        return $config;
    }
    
}