<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Redirect
{
    
    private $_args;
    private $_params;
    private $_request;
    
    public function __construct(&$args)
    {
        $this->_args    = $args;
        $this->_request = $this->_args->getRequest();
        $this->_params  = $this->_request->getParams();
    }
    
    public function redirect()
    {
        // redirect is not allowed in blog sections
        if ($this->isRedirectAllowed()) {
            
            $oRedirect = $this->getRedirector();
            
            $jsHeader = "";
            
            $bAutoRedirect = (
                $this->getConfig()->getRedirectType()
                == Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Config::REDIRECT_TYPE_HTTP
            );
            
            if ($this->_request->getControllerName() == 'detail') {
                $sOrdernumber = $this->_getRedirectProductOrderNumber($this->_params);
                $jsHeader     = ($sOrdernumber['public'])
                    ? $oRedirect->buildScriptItemPublic($sOrdernumber['number'], $bAutoRedirect)
                    : $oRedirect->buildScriptItem($sOrdernumber['number'], $bAutoRedirect);
            } else {
                if ($this->_request->getControllerName() == 'listing') {
                    $jsHeader = $oRedirect->buildScriptCategory($this->_params['sCategory'], $bAutoRedirect);
                } else {
                    if ($this->_request->getControllerName() == 'index') {
                        $jsHeader = $oRedirect->buildScriptShop($bAutoRedirect);
                    } else {
                        $jsHeader = $oRedirect->buildScriptDefault($bAutoRedirect);
                    }
                }
            }
            
            $jsHeader = addcslashes($jsHeader, "\\");
            
            $sContent = $this->_args->getSubject()->Response()->getBody();
            $sHeader  = null;
            if (preg_match("#</head>#i", $sContent, $sHeader)) {
                $sHeader      = $sHeader[0];
                $sReplacement = $jsHeader . "\n" . $sHeader;
                
                $sContent = preg_replace("#$sHeader#i", $sReplacement, $sContent);
                $this->_args->getSubject()->Response()->setBody($sContent);
            }
        }
    }
    
    /**
     * @return ShopgateMobileRedirect
     */
    protected function getRedirector()
    {
        $config    = $this->getConfig();
        $builder   = new ShopgateBuilder($config);
        $oRedirect = $builder->buildRedirect();
        
        return $oRedirect;
    }
    
    /**
     *
     * @return Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Config
     */
    protected function getConfig()
    {
        static $config = null;
        
        if (!$config) {
            $config = new Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Config();
        }
        
        return $config;
    }
    
    /**
     * Return the Article Ordernumber of the given Article
     *
     * @param array $params
     *
     * @return array An array with indices 'public' (bool) and 'number' containing the item_number or item_number_public.
     */
    protected function _getRedirectProductOrderNumber($params)
    {
        $sql   = "SELECT COUNT(*) as count
        FROM s_articles_details sad
        WHERE sad.articleID = '{$params['sArticle']}'";
        $count = Shopware()->Db()->query($sql)->fetchColumn(0);
        
        $item_number = array('public' => false, 'number' => '');
        
        if ($count > 1) {
            /*** eindimensionale Variante ***/
            $item_number['number'] = 'sgParent' . $params['sArticle'];
        } else {
            /*** configurator or normal article ***/
            $sql         = "SELECT sad.ordernumber
                FROM s_articles_details sad
                WHERE sad.articleID = '{$params['sArticle']}'";
            $ordernumber = Shopware()->Db()->query($sql)->fetchColumn(0);
            
            $sql     = "SELECT COUNT(*) as count
                        FROM s_articles_groups_settings
                        WHERE articleID = '{$params['sArticle']}'";
            $counter = Shopware()->Db()->query($sql)->fetchColumn(0);
            
            $sql     = "SELECT type
                FROM s_articles_groups_settings
                WHERE articleID = '{$params['sArticle']}'";
            $result2 = Shopware()->Db()->query($sql)->fetchColumn(0);
            
            /*** normal article / upprice article ***/
            if ($counter == 0 || $result2 == 3) {
                $item_number['public'] = true;
                $item_number['number'] = $ordernumber;
            }
            /*** configurator article ***/
            if ($result2 !== false && $result2 != 3) {
                $item_number['number'] = 'sgParent' . $params['sArticle'];
            }
        }
        
        return $item_number;
    }
    
    protected function isRedirectAllowed()
    {
        if (defined("_SHOPGATE_API") && _SHOPGATE_API) {
            return false;
        }
        
        if (!$this->getConfig()->getIsActive()) {
            return false;
        }
        
        if ($this->_params['module'] == 'backend') {
            return false;
        }
        
        if ($this->_params['sCategory']) {
            $blog = Shopware()->Db()->fetchOne(
                "SELECT blog FROM `s_categories` WHERE id = {$this->_params['sCategory']}"
            )
            ;
            
            if ($blog) {
                return false;
            }
        }
        
        if ($this->_params['sArticle']) {
            $mode = Shopware()->Db()->fetchOne(
                "SELECT mode FROM `s_articles` WHERE id = {$this->_params['sArticle']}"
            )
            ;
            if ($mode) {
                return false;
            }
        }
        
        return true;
    }
    
}