<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

require_once 'shopgate_library/shopgate.php';

class ShopgatePluginShopware extends ShopgatePlugin
{
    
    const STATE_SHIPPING_OPEN              = 0;
    const STATE_SHIPPING_PENDING           = 1;
    const STATE_SHIPPING_COMPLETED         = 2;
    const STATE_SHIPPING_PARTIAL_COMPLED   = 3;
    const STATE_SHIPPING_CANCLED           = 4;
    const STATE_SHIPPING_FREE_FOR_SHIPPING = 5;
    const STATE_SHIPPING_SHIPPED_PARTIAL   = 6;
    const STATE_SHIPPING_SHIPPED_COMPLETED = 7;
    const STATE_SHIPPING_CLARIFY           = 8;
    
    const STATE_PAYMENT_COMPLETE = 12;
    const STATE_PAYMENT_OPEN     = 17;
    
    const MALE   = "mr";
    const FEMALE = "ms";
    
    const PAYMENT_NAME_PAYPAL = "paypalexpress";
    
    /**
     * @var sShopwareExport
     */
    protected $export;
    /**
     * @var unknown_type
     */
    protected $locale;
    /**
     * @var unknown_type
     */
    protected $subShop;
    /**
     * @var unknown_type
     */
    protected $sSYSTEM;
    
    /**
     * @var Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Config
     */
    protected $config;
    
    /*
     * @var array
     */
    protected $shopgateOrderData;
    
    /*
     * @var boolean
     */
    protected $paymentPaypalSupported;
    
    public function startup()
    {
        if (!defined("SHOPGATE_PLUGIN_VERSION")) {
            define("SHOPGATE_PLUGIN_VERSION", Shopware()->Plugins()->Backend()->SgateShopgatePlugin()->getVersion());
        }
        
        $this->export            = Shopware()->Api()->Export();
        $this->locale            = Shopware()->Locale()->getId();
        $this->subShop           = Shopware()->System()->sSubShop;
        $this->sSYSTEM           = Shopware()->System();
        $this->config            = new Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Config();
        $this->shopgateOrderData = null;
        
        $paymenPaypalId = Shopware()->Db()->fetchOne(
            "SELECT id FROM `s_core_paymentmeans` WHERE name = '".(self::PAYMENT_NAME_PAYPAL)."'"
        );
        $this->paymentPaypalSupported = !empty($paymenPaypalId);
        
        return true;
    }
    
    ############################################################################
    ## LOADERS
    ############################################################################
    protected function getPluginInfoLoaders()
    {
        return array(
            "loadShopwareVersion", /** $this->loadShopwareVersion */
        );
    }
    
    protected function getAddOrderLoaders()
    {
        return array(
            'insertShopgateOrderData', /** $this->insertShopgateOrderData */
            'insertOrder', /** $this->insertOrder */
            'insertOrderCustomer', /** $this->insertOrderCustomer */
            'insertOrderDeliveryAddress', /** $this->insertOrderDeliveryAddress */
            'insertOrderInvoiceAddress', /** $this->insertOrderInvoiceAddress */
            'insertOrderCountry', /** $this->insertOrderCountry */
            'insertOrderCountryShipping', /** $this->insertOrderCountryShipping */
            'insertOrderItems', /** $this->insertOrderItems */
            'insertOrderExternalCoupons', /** $this->insertOrderExternalCoupons */
            'insertOrderPaymentCosts', /** $this->insertOrderPaymentCosts */
            'insertOrderPaymentAmount', /** $this->insertOrderPaymentAmount */
            'insertOrderPayment', /** $this->insertOrderPayment */
        );
    }
    
    protected function getAfterAddOrderLoaders()
    {
        return array(
            'setOrderPayment', /** $this->setOrderPayment */
            'setOrderClear', /** $this->setOrderClear */
            'setOrderStatus', /** $this->setOrderStatus */
            'insertPlentyOrderData', /** $this->insertPlentyOrderData */
        );
    }
    
    ############################################################################
    ## CRON                                                                   ##
    ############################################################################
    
    public function cron($jobname, $params, &$message, &$errorcount)
    {
        switch ($jobname) {
            case "set_shipping_completed":
                Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Cron::checkOrderStatus($message, $errorcount);
                break;
            case "cancel_orders":
                $this->cronCancelOrders($message, $errorcount);
                break;
            case "clean_orders":
                Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Cron::cleanupOrders($message, $errorcount);
                break;
            default:
                throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_CRON_UNSUPPORTET_JOB);
        }
    }
    
    protected function cronCancelOrders(&$message, &$errorcount)
    {
        $orders = Shopware()->Db()->fetchAll(
            "SELECT DISTINCT
                so.shopgate_order_number
                FROM s_shopgate_orders so
                JOIN s_order o ON (so.orderID = o.id)
                WHERE so.is_cancellation_sent_to_shopgate = 0
                  AND DATE(o.ordertime) > ADDDATE(NOW(), INTERVAL -4 WEEK)");
        
        $orderComponent = new Shopware_Plugins_Backend_SgateShopgatePlugin_Components_Order();
        
        //$orders = array(array("shopgate_order_number" => "1013000336"));
        
        foreach ($orders as $row) {
            $this->log("Try to set shipping completed: " . print_r($row, true), ShopgateLogger::LOGTYPE_DEBUG);
            try {
                if ($orderComponent->cancelOrder($row["shopgate_order_number"])) {
                    $message .= "Successfull cancled order to #{$row["shopgate_order_number"]}\n";
                }
                
            } catch (Exception $e) {
                echo $e->getTraceAsString();
                
                $errorcount++;
                $message .= "!! Error on cancled order to #{$row["shopgate_order_number"]}\n";
            }
        }
        
    }
    
    
    ############################################################################
    ## PLUGIN-INFOS                                                           ##
    ############################################################################
    
    public function createPluginInfo()
    {
        $aInfos = array();
        $aInfos = $this->executeLoaders($this->getPluginInfoLoaders(), $aInfos);
        
        return $aInfos;
    }
    
    protected function loadShopwareVersion($aInfos)
    {
        $aInfos["shop_version"] = $this->sSYSTEM->sCONFIG["sVERSION"];
        
        return $aInfos;
    }
    
    
    ############################################################################
    ## SHOPGATE-CONNECT                                                       ##
    ############################################################################
    
    /**
     *
     * @param String $user
     * @param String $pass
     *
     * @return ShopgateCustomer
     */
    public function getCustomer($user, $pass)
    {
        $sql = "
            SELECT
                id
            FROM s_user
            WHERE email = '{$user}'
            AND password = '" . md5($pass) . "'
            AND subshopID = '{$this->locale}'
        ";
        $userID = Shopware()->Db()->fetchOne($sql);
        
        if (empty($userID)) {
            throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_WRONG_USERNAME_OR_PASSWORD);
        } else {
            $user = $this->getShopCustomer($userID);
            
            $userData = new ShopgateCustomer();
            
            $userData->setCustomerId($user['userID']);
            $userData->setCustomerNumber($user['customernumber']);
            
            $userData->setGender(
                $user['billing_salutation'] == "mr" ? ShopgateCustomer::MALE : ShopgateCustomer::FEMALE
            );
            $userData->setFirstName($user['billing_firstname']);
            $userData->setLastName($user['billing_lastname']);
            
            $userData->setMail($user['email']);
            $userData->setPhone($user['billing_phoney']);
            
            // Invoice Address
            $oInvoiceAddress = new ShopgateAddress();
            $oInvoiceAddress->setAddressType(ShopgateAddress::INVOICE);
            
            $oInvoiceAddress->setGender(
                $user['billing_salutation'] == "mr" ? ShopgateCustomer::MALE : ShopgateCustomer::FEMALE
            );
            $oInvoiceAddress->setFirstName($user['billing_firstname']);
            $oInvoiceAddress->setLastName($user['billing_lastname']);
            
            $oInvoiceAddress->setCompany($user['billing_company']);
            $oInvoiceAddress->setStreet1($user['billing_street'] . ' ' . $user['billing_streetnumber']);
            $oInvoiceAddress->setZipcode($user['billing_zipcode']);
            $oInvoiceAddress->setCity($user["billing_city"]);
            $oInvoiceAddress->setCountry($user['billing_country_iso']);
            $oInvoiceAddress->setPhone($user['billing_phoney']);
            
            // Shipping Address
            $oShippingAddress = new ShopgateAddress();
            
            if ($user["shipping_firstname"]) {
                $oShippingAddress->setAddressType(ShopgateAddress::DELIVERY);
                $oShippingAddress->setGender(
                    $user['shipping_salutation'] == "mr" ? ShopgateCustomer::MALE : ShopgateCustomer::FEMALE
                );
                $oShippingAddress->setFirstName($user['shipping_firstname']);
                $oShippingAddress->setLastName($user['shipping_lastname']);
                
                $oShippingAddress->setCompany($user['shipping_company']);
                $oShippingAddress->setStreet1($user['shipping_street'] . ' ' . $user['shipping_streetnumber']);
                $oShippingAddress->setZipcode($user['shipping_zipcode']);
                $oShippingAddress->setCity($user["shipping_city"]);
                $oShippingAddress->setCountry($user['shipping_country_iso']);
            } else {
                $oShippingAddress = clone $oInvoiceAddress;
                $oShippingAddress->setAddressType(ShopgateAddress::DELIVERY);
            }
            
            $userData->setAddresses(array($oInvoiceAddress, $oShippingAddress));
            
            $userData = $userData->utf8Encode();
            
            return $userData;
        }
    }
    
    
    ############################################################################
    ## CREATE CATEGRIES CSV                                                   ##
    ############################################################################
    
    protected $iMaxCategoryPosition = 0;
    
    protected function createCategoriesCsv()
    {
        $maxposition = $this->loadMaxCategoryPosition();
        
        $mapping = array(
            'id'          => 'id',
            'parent'      => 'parent',
            'position'    => 'position',
            'description' => 'description',
            'active'      => 'active',
            'childs'      => 'childs',
            'blog'        => 'blog',
        );
        $tree    = $this->export->sCategoryTree($mapping, $this->subShop['parentID']);
        
        foreach ($tree as $category) {
            // Ignore inactive and blog categories
            if (!$category["active"] == 1 || $category["blog"] == 1) {
                continue;
            }
            
            $this->buildCategoriesTree($category, $maxposition);
        }
    }
    
    protected function buildCategoriesTree($category)
    {
        $aRow = array();
        
        $aRow = $this->executeLoaders($this->getCreateCategoriesCsvLoaders(), $aRow, $category);
        
        $this->addCategoryRow($aRow);
        
        if ($category['childs']) {
            foreach ($category['childs'] as $child) {
                if ($child["blog"] == 1) {
                    continue;
                }
                if (!$child["active"] == 1) {
                    continue;
                }
                
                $this->buildCategoriesTree($child);
            }
        }
    }
    
    protected function loadMaxCategoryPosition()
    {
        $sql                        = "SELECT MAX(position) AS maxposition FROM s_categories";
        $this->iMaxCategoryPosition = (Shopware()->Db()->fetchOne($sql) + 1);
        
        return $this->iMaxCategoryPosition;
    }
    
    /**
     * Fill the Field category_number in the given array
     *
     * @return array
     */
    protected function categoryExportCategoryNumber($aItem, $aCategory)
    {
        $aItem["category_number"] = $aCategory["id"];
        
        return $aItem;
    }
    
    /**
     * Fill the Field category_name in the given array
     *
     * @return array
     */
    protected function categoryExportCategoryName($aItem, $aCategory)
    {
        $aItem["category_name"] = $aCategory["description"];
        
        return $aItem;
    }
    
    /**
     * Fill the Field parent_id in the given array
     *
     * @return array
     */
    protected function categoryExportParentId($aItem, $aCategory)
    {
        $iParentId = "";
        if (($aCategory['parent'] != $this->subShop['parentID'])
        && ($aCategory['parent'] != $category['id'])) {
            $iParentId = $aCategory['parent'];
        }
        
        $aItem["parent_id"] = $iParentId;
        
        return $aItem;
    }
    
    /**
     * Fill the Field order_index in the given array
     *
     * @return array
     */
    protected function categoryExportOrderIndex($aItem, $aCategory)
    {
        $aItem["order_index"] = $this->iMaxCategoryPosition - $aCategory['position'];
        
        return $aItem;
    }
    
    /**
     * Fill the Field is_active in the given array
     *
     * @return array
     */
    protected function categoryExportIsActive($aItem, $aCategory)
    {
        $aItem["is_active"] = $aCategory["active"];
        
        return $aItem;
    }
    
    /**
     * <strong>NOTE:</strong> SHOPWARE DOES NOT SUPPORT CATEGORY-IMAGES
     *
     * Fill the Field url_image in the given array
     *
     * @return array
     */
    protected function categoryExportUrlImage($aItem, $aCategory)
    {
        $aItem["url_image"] = "";
        
        return $aItem;
    }
    
    /**
     * Fill the Field url_deeplink in the given array
     *
     * @return array
     */
    protected function categoryExportUrlDeeplink($aItem, $aCategory)
    {
        $sLink = $this->sSYSTEM->sCONFIG['sBASEFILE'] . "?sViewport=cat&sCategory={$aCategory['id']}";
        $sLink = $this->sSYSTEM->sMODULES['sCore']->sRewriteLink($sLink);
        
        $aItem["url_deeplink"] = $sLink;
        
        return $aItem;
    }
    
    
    ############################################################################
    ## CREATE ITEMS CSV                                                       ##
    ############################################################################
    
    protected function getExportArticleIds()
    {
        $categoryIds = array_keys(Shopware()->Api()->Export()->sCategories(array(), $this->subShop['parentID']));
        
        $sql = "
            SELECT DISTINCT a.id as `articleID`
            FROM s_articles AS a
                INNER JOIN s_articles_categories AS ac ON ac.articleID = a.id
            WHERE ac.categoryID in (" . implode(',', $categoryIds) . ") AND {$this->getActiveArticleSqlSnippet()}
        ";
        
        if ($this->splittedExport) {
            $sql .= " LIMIT {$this->exportOffset}, {$this->exportLimit}";
        }
        
        $result = Shopware()->Db()->query($sql);
        
        $ids = array();
        foreach ($result->fetchAll() as $article) {
            $ids[] = array_shift($article);
        }
        
        return $ids;
    }
    
    protected function buildDefaultItemRow()
    {
        $bExportNetto = false;
        
        static $row = null;
        if (empty($row)) {
            $row = parent::buildDefaultItemRow();
            
            if ($bExportNetto) {
                // remove old fileds
                unset($row["unit_amount"]);
                unset($row["old_unit_amount"]);
                unset($row["tax_percent"]);
                
                $newFields = array(
                    "tax_class"           => "",  /** $this->itemExportTaxClass */
                    "unit_amount_net"     => "0", /** $this->itemExportUnitAmountNet */
                    "old_unit_amount_net" => "",  /** $this->itemExportOldUnitAmountNet */
                );
                
                $row = array_slice($row, 0, 3, true) +
                    $newFields +
                    array_slice($row, 3, count($row) - 3, true);
            }
        }
        
        return $row;
    }
    
    protected function createItemsCsv()
    {
        if (!$this->splittedExport || $this->exportOffset == 0) {
            
            $categoryIds = array_keys(Shopware()->Api()->Export()->sCategories(array(), $this->subShop['parentID']));
            
            $sql = "
                SELECT count(DISTINCT a.id) FROM s_articles AS a
                    INNER JOIN s_articles_categories AS ac ON ac.articleID = a.id
                WHERE ac.categoryID in (" . implode(',', $categoryIds) . ") AND {$this->getActiveArticleSqlSnippet()}
            ";
            $count = Shopware()->Db()->fetchOne($sql);
            $this->log("Active Products in Database: {$count}", ShopgateLogger::LOGTYPE_ACCESS);
        }
        
        $articleIds = $this->getExportArticleIds();
        foreach ($articleIds as $articleId) {
            $this->log("## Start export article with ID: {$articleId}", ShopgateLogger::LOGTYPE_DEBUG);
            $article = $this->getArticleById($articleId);
            if (empty($article)) {
                continue;
            }
            
            $items = $this->buildItem($article);
            
            $this->log("## End export article with ID: {$articleId}", ShopgateLogger::LOGTYPE_DEBUG);
            
            if ($items == false) {
                continue;
            }
            
            foreach ($items as $item) {
                $this->addItemRow($item);
            }
        }
        
    }
    
    /**
     * creates a formatted article or a formatted array of articles
     *
     * @param $article
     *
     * @return array
     */
    protected function buildItem($article)
    {
        $children = array();
        
        $sgItem = $this->buildDefaultItemRow();
        $sgItem = $this->executeLoaders($this->getCreateItemsCsvLoaders(), $sgItem, $article);
        
        /*** what kind of article? 											***
         *** normal article 			-> nothing special 					***
         *** one-dimensional variant 	-> build parent & use attribute_1 	***
         *** configurator article 		-> build parent & use attributes 	***
         *** articles using upprices 	-> build article with options 		***/
        
        /*** one-dimensional variant ***/
        if (!empty($article['sVariants'])) {
            
            /*** create parent ***/
            $parent = $this->buildShopgateParentArticle($sgItem, $article);
            
            /*** create children ***/
            $children = $this->buildVariationArticles($sgItem, $article);
            
            /*** merge parent and children ***/
            $sgItems = array_merge(array($parent), $children);
            
            return $sgItems;
        }
        
        /*** Configurator article ***/
        if ($this->checkIfConfig($article['articleID'])) {
            /*** upprice article ***/
            if ($article['sConfiguratorSettings']['type'] == 3) {
                /*** create upprice article using options ***/
                $sgItem = $this->buildUppriceArticle($sgItem, $article);
                
                return array($sgItem);
            }
            
            /*** non upprice configurator article ***/
            /*** create parent ***/
            $parent = $this->buildShopgateConfiguratorParentArticle($sgItem, $article);
            
            /*** create children ***/
            $children = $this->buildConfiguratorArticles($sgItem, $article);
            
            /*** merge parent and children ***/
            $sgItems = array_merge(array($parent), $children);
            
            return $sgItems;
        }
        
        return array($sgItem);
    }
    
    /**
     * Fill the Field item_number in the given array
     *
     * @return array
     */
    protected function itemExportItemNumber($aItem, $aArticle)
    {
        $aItem["item_number"] = $aArticle["ordernumber"];
        
        return $aItem;
    }
    
    /**
     * Fill the Field item_number_public in the given array
     *
     * @return array
     */
    protected function itemExportItemNumberPublic($aItem, $aArticle)
    {
        $aItem["item_number_public"] = $aArticle["ordernumber"];
        
        return $aItem;
    }
    
    /**
     * Fill the Field item_name in the given array
     *
     * @return array
     */
    protected function itemExportItemName($aItem, $aArticle)
    {
        $aItem["item_name"] = $aArticle["articleName"];
        
        if ($aArticle["purchasesteps"]) {
            $aItem["item_name"] .= "  ({$aArticle["purchasesteps"]}er Packung)";
// 			$aItem["item_name"] = "{$aArticle["purchasesteps"]}x {$aItem["item_name"]}";
        }
        
        $aItem["item_name"] = htmlentities($aItem["item_name"]);
        
        return $aItem;
    }
    
    /**
     * Fill the Field unit_amount in the given array
     *
     * @return array
     */
    protected function itemExportUnitAmount($aItem, $aArticle)
    {
        $price = $aArticle["price"];
        
        if ($aArticle["purchasesteps"]) {
            $price *= $aArticle["purchasesteps"];
        }
        
        $aItem["unit_amount"] = $this->formatPriceNumber($price, 2);
        
        return $aItem;
    }
    
    protected function itemExportUnitAmountNet($aItem, $aArticle)
    {
        $price = $aArticle["price"];
        
        if ($aArticle["purchasesteps"]) {
            $price *= $aArticle["purchasesteps"];
        }
        
        $tax = $aArticle["tax"];
        
        $price                    = $price / (1 + ($tax / 100));
        $aItem["unit_amount_net"] = $this->formatPriceNumber($price, 2);
        
        return $aItem;
    }
    
    protected function itemExportOldUnitAmount($aItem, $aArticle)
    {
        $fPrice = 0;
        
        if ($aArticle["pseudoprice"] > 0) {
            $fPrice = $aArticle["pseudoprice"] * (1 + ($aArticle["tax"] / 100));
        }
        
        if ($aArticle["purchasesteps"]) {
            $fPrice *= $aArticle["purchasesteps"];
        }
        
        $aItem["old_unit_amount"] = $this->formatPriceNumber($fPrice, 2);
        
        return $aItem;
    }
    
    protected function itemExportOldUnitAmountNet($aItem, $aArticle)
    {
        $fPrice = 0;
        
        if ($aArticle["pseudoprice"] > 0) {
            $fPrice = $aArticle["pseudoprice"] * (1 + ($aArticle["tax"] / 100));
        }
        
        if ($aArticle["purchasesteps"]) {
            $fPrice *= $aArticle["purchasesteps"];
        }
        
        $tax = $aArticle["tax"];
        
        $fPrice                       = $fPrice / (1 + ($tax / 100));
        $aItem["old_unit_amount_net"] = $this->formatPriceNumber($fPrice, 2);
        
        return $aItem;
    }
    
    /**
     * Fill the Field currency in the given array
     *
     * @return array
     */
    protected function itemExportCurrency($aItem, $aArticle)
    {
        $aItem["currency"] = $this->sSYSTEM->sCurrency['currency'];
        
        return $aItem;
    }
    
    protected function itemExportShippingCostsPerOrder($aItem, $aArticle)
    {
        if ($this->config->getExportPremiumShippingCosts()) {
            $aArticleNew = array(
                'articleID'    => $aArticle['articleID'],
                'ordernumber'  => $aArticle['ordernumber'],
                'shippingfree' => $aArticle['shippingfree'],
                'price'        => $aArticle['price'],
                'netprice'     => $aArticle['netprice'],
                'esd'          => 0,
            );
            
            $aPayment = array('id' => 5); // Vorkasse
            $aCountry = array('id' => $this->subShop["subshopID"]);
            
            $aItem["shipping_costs_per_order"] =
                $this->getArticlePremiumShippingcosts($aArticleNew, $aPayment, $aCountry);
            
            if ($aItem["shipping_costs_per_order"] == '') {
                $aItem["shipping_costs_per_order"] = 0;
            }
        }
        
        return $aItem;
    }
    
    /**
     * Fill the Field tax_percent in the given array
     *
     * @return array
     */
    protected function itemExportTaxPercent($aItem, $aArticle)
    {
        $aItem["tax_percent"] = $aArticle["tax"];
        
        return $aItem;
    }
    
    protected function itemExportTaxClass($aItem, $aArticle)
    {
        $aItem["tax_class"] = "{$aArticle["taxID"]}=>tax_{$aArticle["tax"]}";
        
        return $aItem;
    }
    
    /**
     * Fill the Field description in the given array
     *
     * @return array
     */
    protected function itemExportDescription($aItem, $aArtcile)
    {
        $description = $aArtcile["description_long"];
        
        $_properties = $this->buildAttributesProperties($aArtcile, array("textarea"));
        foreach ($_properties as $key => $value) {
            $description .= "<br /><h2>{$key}</h2>{$value}";
        }
        
        $aItem["description"] = $description;
        
        return $aItem;
    }
    
    /**
     * Fill the Field urls_images in the given array
     *
     * @return array
     */
    protected function itemExportUrlsImages($aItem, $aArticle)
    {
        $aImages = array();
        
        // TODO: Refactor images export functionality
        
        if (!empty($aArticle['image'])) {
            $aImages[] = $aArticle['image']['src']['original'];
        }
        
        if (!empty($aArticle['images'])) {
            foreach ($aArticle['images'] as $img) {
                // take care of variant images / filter other variant images
                if (!empty($img['relations']) && $aItem['item_number'] == $img['relations']
                    || empty($img['relations'])
                ) {
                    $aImages[] = $img['src']['original'];
                }
            }
        }
        $aItem["urls_images"] = implode("||", $aImages);
        
        return $aItem;
    }
    
    /**
     * Builds a filesystem path to the shop root directory and returns it
     *
     * @return string
     */
    protected function getShopRootFS()
    {
        return DS . trim(
            str_replace(
                'engine'.DS.'Shopware'.DS.'Plugins'.DS.'Community'.DS.'Backend'.DS.'SgateShopgatePlugin',
                '',
                str_replace( // convert backslashes to normal slashes, and remove double forward slashes
                    array('\\', DS . DS),
                    array(DS, DS),
                    dirname(__FILE__) // take the actual file to get the filesystem path
                )
            ),
            DS
        ) . DS;
    }
    
    /**
     * Builds a webserver url to the shop root directory and returns it
     *
     * @return string
     */
    protected function getShopRootWS()
    {
        return ("http:" . DS . DS .
            trim(
                str_replace( // take host path without "http"
                    'http:' . DS . DS,
                    '',
                    Shopware()->Shop()->getHost()
                ),
                DS
            ) .
            trim(DS . Shopware()->Shop()->getBasePath(), DS) // append the path to shop location
        ) .
        DS;
    }
    
    /**
     * Takes an absolute filesystem path or url (or alreay relative path) and converts it to a relative path by removing the fs or ws root dir.
     * Pre and post slashes are removed if there are any
     *
     * @param string $absolutePath
     *
     * @return string
     */
    protected function getRelativePath($absolutePath)
    {
        if (($httppos	= strpos($absolutePath, 'http://')	!== FALSE && $httppos	== 0)
        || ($httpspos	= strpos($absolutePath, 'https://')	!== FALSE && $httpspos	== 0)) {
            // URL given
            $rootDir = $this->getShopRootWS();
        } elseif ($fsrootpos = strpos($absolutePath, trim($rootDir = $this->getShopRootFS(), DS)) !== FALSE && $fsrootpos == 0) {
            // fs root matched
            // $rootDir has already been assigned, since getShopRootFS has already been called and directly saved to $rootDir while testing for match
        } else {
            // no match, assume its already relative
            $rootDir = $absolutePath;
        }
        
        return trim(str_replace($rootDir, '', $absolutePath), DS);
    }
    
    /**
     * Takes a set of an image path including the cached images and takes the best available one (prefers the original if available)
     *
     * @param array $imgSrcArray
     *
     * @return string
     */
    protected function getSingleArticleImage($imgSrcArray)
    {
        $image = '';
        if (!empty($imgSrcArray)) {
            if (!file_exists($this->getShopRootFS() . $this->getRelativePath($imgSrcArray['original']))) {
                // check the thumbnail pictures for existence in case the original has been deleted
                $checkIndex = count($imgSrcArray) - 1; // subtract an additional one because of the "original" key entry
                
                while (--$checkIndex >= 0) {
                    // find the first available image in list (the biggest images are located at the end so check backwards)
                    if (file_exists($this->getShopRootFS() . $this->getRelativePath($imgSrcArray[$checkIndex]))) {
                        $image = $this->getShopRootWS() . $this->getRelativePath($imgSrcArray[$checkIndex]);
                        break;
                    }
                }
            } else {
                $image = $this->getShopRootWS() . $this->getRelativePath($imgSrcArray['original']);
            }
        }
        
        return $image;
    }
    
    /**
     * Fill the Field categories in the given array
     *
     * @return array
     */
    protected function itemExportCategories($aItem, $aArticle)
    {
        $aPaths = array();
        
        $categories = $this->getCategoryIds($aArticle['articleID']);
        foreach ($categories as $category) {
            $sPath = $this->getCategoryPath($category);
            if (!empty($sPath)) {
                $aPaths[] = $sPath;
            }
        }
        
        $aItem["categories"] = implode('||', $aPaths);
        
        return $aItem;
    }
    
    /**
     * Fill the Field category_numbers in the given array
     *
     * @return array
     */
    protected function itemExportCategoryNumbers($aItem, $aArticle)
    {
        $categoryIDs               = $this->getCategoryIds($aArticle['articleID']);
        $aItem["category_numbers"] = implode('||', $categoryIDs);
        
        return $aItem;
    }
    
    /**
     * Fill the Field is_available in the given array
     *
     * @return array
     */
    protected function itemExportIsAvailable($aItem, $aArticle)
    {
        $aItem["is_available"] = 1;
        
        return $aItem;
    }
    
    /**
     * Fill the Field available_text in the given array
     *
     * @return array
     */
    protected function itemExportAvailableText($aItem, $aArticle)
    {
        $sAvailableText = "";
        
        $articleReleaseDate = (!empty($aArticle['releasedate'])) ? $aArticle['releasedate'] : "";
        $articleEsd         = $aArticle['esd'];
        $articleInStock     = $aArticle['instock'];
        $shippingTime       = $aArticle['shippingtime'];
        
        if ($articleReleaseDate && date('Ymd', strtotime($articleReleaseDate)) > date('Ymd')) {
            $sAvailableText = $this->getTemplateText("DetailDataInfoShipping") . ' ' . date('d.m.Y', strtotime($articleReleaseDate));
        } elseif ($articleEsd) {
            $sAvailableText = $this->getTemplateText("DetailDataInfoInstantDownload");
        } elseif ($articleInStock > 0) {
            $sAvailableText = $this->getTemplateText("DetailDataInfoInstock");
        } elseif ($shippingTime) {
            $sAvailableText = $this->getTemplateText("DetailDataShippingtime") . " {$shippingTime} "  . $this->getTemplateText("DetailDataShippingDays");
        } else {
            $sAvailableText = Shopware()->Db()->fetchOne("SELECT value FROM `s_core_config` WHERE name = 'sNOTAVAILABLE'");
        }
        
        $matches = null;
        // See sample #3 (named subpatterns) of preg_match_all for details of array structure
        // http://de2.php.net/manual/de/function.preg-match-all.php
        preg_match_all('/\{(?P<object>\$.+)\.(?P<field>.+)\}/i', $sAvailableText, $matches);
        for ($i = 0; isset($matches["field"][$i]); $i++) {
            $field          = $matches["field"][$i];
            $regex          = preg_quote($matches[0][$i]);
            $sAvailableText = preg_replace("/{$regex}/i", $aArticle[$field], $sAvailableText);
        }
        
        $aItem["available_text"] = $sAvailableText;
        
        return $aItem;
    }
    
    protected function itemExportBasicPrice($aItem, $aArticle)
    {
        if ($aArticle["purchaseunit"] > 0 && !empty($aArticle["referenceunit"])) {
            $sBasicUnit = Shopware()->Db()->fetchOne(
                "SELECT description FROM `s_core_units` WHERE id = {$aArticle["unitID"]}"
            );
            $sBasicPrice = $aArticle["price"] / $aArticle["purchaseunit"] * $aArticle["referenceunit"];
            $sBasicPrice = $this->formatPriceNumber($sBasicPrice, 2);
            $sCurrencySymbol = $this->sSYSTEM->sCurrency["templatechar"];
            
            if ($this->sSYSTEM->sCurrency["symbol_position"] == 32) { // Symbol on Left side
                $sBasicPrice = $this->sSYSTEM->sCurrency["templatechar"] . $sBasicPrice;
            } else { // Symbol on right side
                $sBasicPrice = $sBasicPrice . " " . $this->sSYSTEM->sCurrency["templatechar"];
            }
            
            $aItem["basic_price"] = htmlentities("{$sBasicPrice} pro {$aArticle["referenceunit"]} {$sBasicUnit}", ENT_QUOTES, "cp1252");
        }
        
        return $aItem;
    }
    
    /**
     * Fill the Field url_deeplink in the given array
     *
     * @return array
     */
    protected function itemExportUrlDeeplink($aItem, $aArticle)
    {
        $aItem["url_deeplink"] = $aArticle["linkDetailsRewrited"];
        
        return $aItem;
    }
    
    /**
     * Fill the Field properties in the given array
     *
     * @return array
     */
    protected function itemExportProperties($aItem, $aArticle)
    {
        $properties = array();
        $iEanNumber = 0;
        
        if ($this->config->getEanAttributeNumber()) {
            $iEanNumber = $this->config->getEanAttributeNumber();
        }
        
        if ($aArticle["purchaseunit"] > 0) {
            $sBasicUnit = Shopware()->Db()->fetchOne(
                "SELECT description FROM `s_core_units` WHERE id = {$aArticle["unitID"]}"
            );
            $properties[] = "Inhalt=>{$aArticle["purchaseunit"]} {$sBasicUnit}";
        }
        
        if ($aArticle["purchasesteps"]) {
            $properties[] = "Menge=>{$aArticle["purchasesteps"]}er Packung";
        }
        
        $_properties = $this->buildAttributesProperties($aArticle, array("text", "boolean"));
        foreach ($_properties as $key => $value) {
            $key   = strip_tags($key);
            $value = strip_tags($value);
            
            $properties[] = "{$key}=>{$value}";
        }

// 		if ($this->config->getExportProperties()) {
        $_properties = $this->buildProperties($aArticle['filtergroupID'], $aArticle['articleID']);
        $properties  = array_merge($properties, $_properties);
// 		}
        
        $aItem["properties"] = implode("||", $properties);
        
        return $aItem;
    }
    
    protected function buildAttributesProperties(array $aArticle, array $type)
    {
        $properties = array();
        $iEanNumber = 0;
        
        if ($this->config->getEanAttributeNumber()) {
            $iEanNumber = $this->config->getEanAttributeNumber();
        }
        
        $sql = "
            SELECT *
            FROM `s_articles_attributes`
            WHERE articleID = '{$aArticle["articleID"]}'
            AND articledetailsID = '{$aArticle["articleDetailsID"]}'
        ";
        $attributes = Shopware()->Db()->fetchRow($sql);
        
        $sql = "
            SELECT *
            FROM `s_core_engine_elements`
            WHERE `group` = 7
            AND `domtype` IN ('".implode("','", $type)."')
            ORDER BY `position`
        ";
        $result = Shopware()->Db()->query($sql);
        while ($element = $result->fetch()) {
            if ("attr{$iEanNumber}" == $element["databasefield"]) {
                continue;
            }
            
            $idsToExport = $this->config->getExportAttributeIds(true);
            foreach ($idsToExport as $attributeId) {
                if ("attr{$attributeId}" == $element["databasefield"]) {
                    $attribute = $attributes[$element["databasefield"]];
                    
                    if (!empty($attribute) && $attribute != "0000-00-00") {
                        $properties[$element["domdescription"]] = $attribute;
                    }
                }
            }
        }
        
        return $properties;
    }
    
    protected function itemExportIsHighlight($aItem, $aArticle)
    {
        $aItem["is_highlight"] = $aArticle["topseller"];
        
        return $aItem;
    }
    
    /**
     * Fill the Field ean in the given array
     *
     * @return array
     */
    protected function itemExportEan($aItem, $aArticle)
    {
        if ($this->config->getEanAttributeNumber()) {
            $ean = $aArticle['attr' . $this->config->getEanAttributeNumber()];
        } else {
            $ean = '';
        }
        
        $aItem["ean"] = $ean;
        
        return $aItem;
    }
    
    /**
     * Fill the Field stock_quantity in the given array
     *
     * @return array
     */
    protected function itemExportUseStock($aItem, $aArticle)
    {
        $aItem["use_stock"] = ((int)$aArticle["laststock"]);
        
        return $aItem;
    }
    
    /**
     * Fill the Field active_status in the given array
     *
     * @param $aItem
     * @param $aArticle
     *
     * @return mixed
     */
    protected function itemExportActiveStatus($aItem, $aArticle)
    {
        
        $aItem["active_status"] = ((int)$aArticle["laststock"]) ? "active" : "stock";
        
        return $aItem;
    }
    
    /**
     * Fill the Field stock_quantity in the given array
     *
     * @return array
     */
    protected function itemExportStockQuantity($aItem, $aArticle)
    {
        $stock = $aArticle["instock"];
        
        if ($aArticle["purchasesteps"]) {
            $stock = floor($stock / $aArticle["purchasesteps"]);
        }
        
        $aItem["stock_quantity"] = $stock;
        
        return $aItem;
    }
    
    protected function itemExportMinimumOrderQuantity($aItem, $aArticle)
    {
        $aItem["minimum_order_quantity"] = $aArticle["minpurchase"];
        
        if ($aArticle["purchasesteps"]) {
            $aItem["minimum_order_quantity"] = ceil($aArticle["minpurchase"] / $aArticle["purchasesteps"]);
        }
        
        return $aItem;
    }
    
    protected function itemExportMaximumOrderQuantity($aItem, $aArticle)
    {
        $aItem["maximum_order_quantity"] = $aArticle["maxpurchase"];
        if ($aArticle["purchasesteps"]) {
            $aItem["maximum_order_quantity"] = floor($aArticle["maxpurchase"] / $aArticle["purchasesteps"]);
        }
        
        return $aItem;
    }
    
    /**
     * Fill the Field manufacturer in the given array
     *
     * @return array
     */
    protected function itemExportManufacturer($aItem, $aArticle)
    {
        $aItem["manufacturer"] = $aArticle["supplierName"];
        
        return $aItem;
    }
    
    /**
     * Fill the Field manufacturer_item_number in the given array
     *
     * @return array
     */
    protected function itemExportManufacturerItemNumber($aItem, $aArticle)
    {
        $manItemNumber = $this->getManufacturerItemNumber($aArticle['articleDetailsID']);
        
        $aItem["manufacturer_item_number"] = $manItemNumber;
        
        return $aItem;
    }
    
    /**
     * Fill the Field last_update in the given array
     *
     * @return array
     */
    protected function itemExportLastUpdate($aItem, $aArticle)
    {
        $aItem["last_update"] = $aArticle["datum"];
        
        return $aItem;
    }
    
    /**
     * Fill the Field tags in the given array
     *
     * @return array
     */
    protected function itemExportTags($aItem, $aArticle)
    {
        $aItem["tags"] = $aArticle["keywords"];
        
        return $aItem;
    }
    
    /**
     * Fill the Field is_free_shipping in the given array
     *
     * @return array
     */
    protected function itemExportIsFreeShipping($aItem, $aArticle)
    {
        $aItem["is_free_shipping"] = $aArticle["shippingfree"];
        
        return $aItem;
    }
    
    /**
     * Fill the Field weight in the given array
     *
     * @return array
     */
    protected function itemExportWeight($aItem, $aArticle)
    {
        $weight          = $aArticle['weight'] * 1000;
        $aItem["weight"] = $weight;
        
        return $aItem;
    }
    
    /**
     * Fill the Field related_shop_items in the given array
     *
     * @param $aItem
     * @param $aArticle
     *
     * @return mixed
     */
    protected function itemExportRelatedShopItems($aItem, $aArticle)
    {
        $result = array();
        
        foreach ($aArticle['sRelatedArticles'] as $article) {
            $result[] = array(
                'type'        => 'upsell',
                'item_number' => $article['relatedarticle'],
                'restricted'  => false
            );
        }
        
        foreach ($aArticle['sSimilarArticles'] as $article) {
            $result[] = array(
                'type'        => 'upsell',
                'item_number' => $article['relatedarticle'],
                'restricted'  => false
            );
        }
        
        if (!empty($result)) {
            $aItem['related_shop_items'] = $this->jsonEncode($result);
        }
        
        return $aItem;
    }
    
    /**
     * Fill the Field internal_order_info in the given array
     *
     * @return array
     */
    protected function itemExportInternalOrderInfo($aItem, $aArticle)
    {
        $useStock = true;
        $useStock = $this->sSYSTEM->sCONFIG['sDEACTIVATENOINSTOCK'] == 1;
        $useStock = $useStock && $aArticle["laststock"] == 1;
        
        $infos                  = array();
        $infos["articleID"]     = $aArticle['articleID'];
        $infos["tax_id"]        = $aArticle['taxID'];
        $infos["use_stock"]     = $useStock;
        $infos["purchasesteps"] = $aArticle["purchasesteps"];
        
        $aItem["internal_order_info"] = $this->jsonEncode($infos);
        
        return $aItem;
    }
    
    protected function itemExportBlockPricing($aItem, $aArticle)
    {
        if (!$aArticle["articleDetailsID"]) {
            return $aItem;
        }
        
        $sql    = "
            SELECT DISTINCT
                ap.from,
                ap.to,
                ap.price
            FROM s_articles_prices ap
            WHERE ap.articleID = {$aArticle["articleID"]}
              AND ap.articledetailsID = {$aArticle["articleDetailsID"]}
              AND ap.pricegroup = 'EK'
            ORDER BY ap.from ASC
        ";
        $result = Shopware()->Db()->fetchAll($sql);
        
        $prices = array();
        foreach ($result as $res) {
            $from  = $res["from"];
            $price = $res["price"];
            $price = $price * (1 + ($aArticle["tax"] / 100));
            $price = $this->formatPriceNumber($price, 2);
            
            $prices[] = "{$from}=>{$price}";
        }
        
        if (count($prices) > 1) {
            $aItem["block_pricing"] = implode("||", $prices);
        }
        
        return $aItem;
    }
    
    
    ############################################################################
    ## EXPORT HELPER                                                          ##
    ############################################################################
    
    protected function getTemplateText($sName, $sNamespace = null)
    {
        $sql = "
            SELECT value FROM s_core_snippets
            WHERE name = '{$sName}' AND shopID = {$this->subShop["id"]} AND localeID = {$this->locale}
        ";
        
        $sText = Shopware()->Db()->fetchOne($sql);
        
        if (!$sText) {
            $sql = "
                SELECT value FROM s_core_snippets
                WHERE name = '{$sName}' AND localeID = {$this->locale}
            ";
            
            if (!empty($sNamespace)) {
                $sql .= " AND namespace = '{$sNamespace}'";
            }
            
            $sText = Shopware()->Db()->fetchOne($sql);
        }
        
        return $sText;
    }
    
    /**
     * Create Properties for articles
     *
     * @param unknown_type $filtergroupID
     * @param unknown_type $articleID
     *
     * @return array ( NAME => VALUE )
     */
    protected function buildProperties($filtergroupID, $articleID)
    {
        $sql    = "
            SELECT DISTINCT
                sfr.id,
                sfr.optionID,
                sfo.name,
                sfv.value
            FROM s_filter_relations sfr
            JOIN s_filter_options sfo ON(sfr.optionID = sfo.id)
            JOIN s_filter_values sfv ON(sfr.optionID = sfv.optionID
                                     AND sfv.groupID = {$filtergroupID}
                                     AND sfv.articleID = {$articleID})
            WHERE sfr.groupID = {$filtergroupID}
            ORDER BY sfr.optionID ASC
        ";
        $result = Shopware()->Db()->fetchAll($sql);
        
        $properties = array();
        foreach ($result as $res) {
            $properties[] = $res['name'] . "=>" . $res['value'];
        }
        
        return $properties;
    }
    
    /*** get all categories of the article ***/
    protected function getCategoryIds($articleID)
    {
        $aCatIds = array();
        
        if (!empty($articleID)) {
            $sql = "
                SELECT DISTINCT
                    c.id
                FROM s_articles_categories ac
                JOIN s_categories c ON(ac.categoryID = c.id)
                WHERE ac.articleID = '{$articleID}'
                    AND ac.categoryID != '{$this->subShop['parentID']}'
                    AND c.active = 1 AND c.blog = 0
            ";
            
            $result = Shopware()->Db()->query($sql);
            while ($row = $result->fetch()) {
                $index     = $this->getArticleOrderIndex($articleID, $row['id']);
                $aCatIds[] = $row['id'] . ($index ? "=>" . $index : "");
            }
        }
        
        return $aCatIds;
    }
    
    protected function getArticleOrderIndex($articleId, $categoryId)
    {
        static $map = array();
        
        if (!$map[$categoryId]) {
            /**
             * sArticle::sGetArticlesByCategory()
             *
             * @var unknown
             */
            Shopware()->System()->_GET['sPerPage'] = 99999;
            //$articles = Shopware()->Modules()->Articles()->sGetArticlesByCategory($categoryId);
            $articles = $this->getArticelIdsByCategoryId($categoryId);
            //$articles = $articles["sArticles"];
            
            $maxSort = count($articles) + 1;
            $i       = 0;
            foreach ($articles as $article) {
                $artId                    = $article["articleID"];
                $map[$categoryId][$artId] = ($maxSort - $i++);
            }
        }
        
        if ($map[$categoryId] && $map[$categoryId][$articleId]) {
            return $map[$categoryId][$articleId];
        }
        
        return null;
    }
    
    protected function getArticelIdsByCategoryId($categoryId)
    {
        $orderBy     = Shopware()->Modules()->Articles()->sSYSTEM->sCONFIG['sORDERBYDEFAULT'] . ', a.id';
        $userGroupId = Shopware()->Modules()->Articles()->sSYSTEM->sUSERGROUPDATA["id"];
        
        if (strpos($orderBy, 'price') !== false) {
            $selectPrice = "
                (IFNULL((
                    SELECT IFNULL(gp.price,gp2.price) as min_price
                    FROM s_articles_groups_value v

                    LEFT JOIN s_articles_groups_settings s
                    ON s.articleID=v.articleID

                    LEFT JOIN s_articles_groups_prices gp
                    ON gp.valueID=v.valueID
                    AND gp.groupkey='{Shopware()->Modules()->Articles()->->sSYSTEM->sUSERGROUP}'
                    AND gp.price!=0

                    LEFT JOIN s_articles_groups_prices gp2
                    ON gp2.valueID=v.valueID
                    AND gp2.groupkey='EK'
                    AND gp2.price!=0

                    WHERE v.active=1
                    AND (s.instock IS NULL OR s.instock!=1 OR v.instock>0)
                    AND IFNULL(gp.price,gp2.price) IS NOT NULL
                    AND v.articleID=a.id

                    ORDER BY min_price
                    LIMIT 1
                ),(
                    SELECT IFNULL(p.price,p2.price) as min_price
                    FROM s_articles_details d

                    LEFT JOIN s_articles_prices p
                    ON p.articleDetailsID=d.id
                    AND p.pricegroup='{Shopware()->Modules()->Articles()->->sSYSTEM->sUSERGROUP}'
                    AND p.to='beliebig'

                    LEFT JOIN s_articles_prices p2
                    ON p2.articledetailsID=d.id
                    AND p2.pricegroup='EK'
                    AND p2.to='beliebig'

                    WHERE d.articleID=a.id

                    ORDER BY min_price
                    LIMIT 1
                )) * 100/(100-IFNULL(cd.discount,0)))
            ";
            $joinPrice   = "
                LEFT JOIN s_core_customergroups cg
                ON cg.groupkey = '{Shopware()->Modules()->Articles()->->sSYSTEM->sUSERGROUP}'

                LEFT JOIN s_core_pricegroups_discounts cd
                ON a.pricegroupActive=1
                AND cd.groupID=a.pricegroupID
                AND cd.customergroupID=cg.id
                AND cd.discountstart=(
                    SELECT MAX(discountstart)
                    FROM s_core_pricegroups_discounts
                    WHERE groupID=a.pricegroupID AND cd.customergroupID=cg.id)
            ";
        } else {
            $selectPrice = 'IFNULL(p.price,p2.price)';
            $joinPrice   = '';
        }
        
        
        $sql = "
            SELECT
                a.id as articleID,aDetails.id AS articleDetailsID, weight,aDetails.ordernumber,a.datum,releasedate,
                additionaltext, shippingfree,shippingtime,instock, a.description AS description, description_long,
                aSupplier.name AS supplierName, aSupplier.img AS supplierImg, a.name AS articleName, topseller as highlight,
                attr1,attr2,attr3,attr4,attr5,attr6,attr7,attr8,attr9,attr10,
                attr11,attr12,attr13,attr14,attr15,attr16,attr17,attr18,attr19,attr20,
                $selectPrice as price
            FROM s_articles_categories AS aCategories

            JOIN s_articles_details AS aDetails
            ON aDetails.articleID=aCategories.articleID
            AND aDetails.kind=1

            JOIN s_articles_attributes AS aAttributes
            ON aAttributes.articleID = aCategories.articleID
            AND aAttributes.articledetailsID=aDetails.id

            JOIN s_articles AS a
            ON a.id=aCategories.articleID

            LEFT JOIN s_articles_supplier AS aSupplier
            ON aSupplier.id=a.supplierID

            LEFT JOIN s_articles_prices p
            ON p.articleDetailsID=aDetails.id
            AND p.pricegroup='{$userGroupId}'
            AND p.to='beliebig'

            LEFT JOIN s_articles_prices p2
            ON p2.articledetailsID=aDetails.id
            AND p2.pricegroup='EK'
            AND p2.to='beliebig'

            AND a.mode = 0
            AND a.active=1
            $joinPrice
            WHERE aCategories.categoryID={$categoryId}
            
            AND (
                SELECT articleID 
                FROM s_articles_avoid_customergroups 
                WHERE articleID = a.id AND customergroupID = {$userGroupId}
            ) IS NULL
            
            ORDER BY {$orderBy}
        ";
        
        return Shopware()->Db()->fetchAll($sql);
    }
    
    /**
     * Build and return the category-paths
     *
     * @param int $articleID
     *
     * @return string
     */
    protected function getCategories($articleID)
    {
        $path = array();
        
        $categories = $this->getCategoryIds($articleID);
        
        foreach ($categories as $category) {
            $return = $this->getCategoryPath($category);
            if (!empty($return)) {
                $path[] = $return;
            }
        }
        
        $path = implode('||', $path);
        
        return $path;
    }
    
    /**
     * Build the category-path recursivly
     *
     * @param int $categoryID
     *
     * @return string
     */
    protected function getCategoryPath($categoryID)
    {
        if (empty($categoryID)) {
            return;
        }
        
        $subShopParent = $this->subShop['parentID'];
        $path          = '';
        
        $result = $this->getCategoryDetails($categoryID);
        
        $path = $result['description'];
        
        if (!empty($result['parent']) && $result['parent'] != 1 && $result['parent'] != $subShopParent) {
            $return = $this->getCategoryPath($result['parent']);
            if ($return != false) {
                $path = $return . (!empty($path) ? '=>' : '') . $path;
            } else {
                return false;
            }
        }
        
        if (($result['parent'] == 1) && ($result['id'] != $subShopParent)) {
            return false;
        }
        
        return $path;
    }
    
    /**
     * get more details for the category
     *
     * id, parent, description
     *
     * @param int $categoryID
     *
     * @return array
     */
    protected function getCategoryDetails($categoryID)
    {
        $sql    = "
            SELECT
                c.id,
                c.parent,
                c.description
            FROM s_categories c
            WHERE c.id = '{$categoryID}'
        ";
        $result = Shopware()->Db()->fetchRow($sql);
        
        return $result;
    }
    
    /*** get manufacturer item number ***/
    protected function getManufacturerItemNumber($detailsID)
    {
        $sql    = "
            SELECT
                suppliernumber
            FROM s_articles_details
            WHERE id = '" . $detailsID . "'
        ";
        $result = Shopware()->Db()->fetchOne($sql);
        
        return $result;
    }
    
    protected function getArticlePremiumShippingcosts($article, $payment, $country, $dispatch = null)
    {
        $basket = $this->getDispatchBasket($article, $country['id'], $payment['id']);
        if (empty($basket)) {
            return false;
        }
        
        $dispatch = $this->getPremiumDispatch($basket, $dispatch);
        
        if (empty($dispatch)) {
            return false;
        }
        
        if ((!empty($dispatch['shippingfree'])&&$dispatch['shippingfree']<=$basket['amount'])
        || (empty($basket['count_article']))
        || (!empty($basket['shippingfree'])&&empty($dispatch['bind_shippingfree']))
        ) {
            if (empty($dispatch['surcharge_calculation'])) {
                return $payment['surcharge'];
            } else {
                return 0;
            }
        }
        
        if (empty($dispatch['calculation'])) {
            $from = round($basket['weight'], 3);
        } elseif ($dispatch['calculation'] == 1) {
            $from = round($basket['amount'], 2);
        } elseif ($dispatch['calculation'] == 2) {
            $from = round($basket['count_article']);
        } elseif ($dispatch['calculation'] == 3) {
            $from = round($basket['calculation_value_' . $dispatch['id']]);
        } else {
            return false;
        }
        
        $sql    = "
            SELECT `value` , `factor`
            FROM `s_premium_shippingcosts`
            WHERE `from`<=$from
                AND `dispatchID`={$dispatch['id']}
            ORDER BY `from` DESC
            LIMIT 1
        ";
        $result = Shopware()->Db()->fetchRow($sql);
        if (empty($result)) {
            return false;
        }
        
        $result['shippingcosts'] = $result['value'];
        if (!empty($result['factor'])) {
            $result['shippingcosts'] += $result['factor'] / 100 * $from;
        }
        $result['surcharge'] = $this->getPremiumDispatchSurcharge($basket);
        if (!empty($result['surcharge'])) {
            $result['shippingcosts'] += $result['surcharge'];
        }
        $result['shippingcosts'] = round($result['shippingcosts'], 2);
        if (!empty($payment['surcharge']) && $dispatch['surcharge_calculation'] != 2
            && (empty($article['shippingfree'])
                || empty($dispatch['surcharge_calculation']))
        ) {
            $result['shippingcosts'] += $payment['surcharge'];
        }
        
        return $result['shippingcosts'];
    }
    
    protected function getDispatchBasket($article, $countryID = null, $paymentID = null)
    {
        $sql_select = '';
        if (!empty(Shopware()->Config()->sPREMIUMSHIPPIUNGASKETSELECT)) {
            $sql_select .= ', ' . Shopware()->Config()->sPREMIUMSHIPPIUNGASKETSELECT;
        }
        $sql          = 'SELECT id, calculation_sql FROM s_premium_dispatch WHERE calculation=3';
        $calculations = Shopware()->Db()->fetchAll($sql);
        
        if (!empty($calculations)) {
            for ($i = 0; $i < count($calculations); $i++) {
                $dispatchID  = $calculations[$i]['id'];
                $calculation = $calculations[$i]['calculation_sql'];
                $sql_select .= ', (' . $calculation . ') as calculation_value_' . $dispatchID;
            }
        }
        
        $sql    = "
            SELECT
                MIN(IFNULL(g.instock,d.instock)>=b.quantity) as instock,
                MIN(IFNULL(g.instock,d.instock)>=(b.quantity+d.stockmin)) as stockmin,
                MIN(a.laststock) as laststock,
                SUM(d.weight*b.quantity) as weight,
                SUM(IF(a.id,b.quantity,0)) as count_article,
                MAX(b.shippingfree) as shippingfree,
                SUM(IF(b.modus=0,b.quantity*b.price/b.currencyFactor,0)) as amount,
                MAX(t.tax) as max_tax,
                u.id as userID
                $sql_select
                ,b.articleID
            FROM (
                SELECT
                NULL as sessionID,
                ? as articleID,
                ? as ordernumber,
                ? as shippingfree,
                1 as quantity,
                ? as price,
                ? as netprice,
                0 as modus,
                ? as esdarticle,
                '' as config,
                ? as currencyFactor,
                '' as ob_attr1,
                '' as ob_attr2,
                '' as ob_attr3,
                '' as ob_attr4,
                '' as ob_attr5,
                '' as ob_attr6
                ) as b
            LEFT JOIN s_articles a ON b.articleID=a.id
                AND b.modus=0
                AND b.esdarticle=0
            LEFT JOIN s_articles_groups_value g
                ON g.ordernumber=b.ordernumber
                AND g.articleID=a.id
            LEFT JOIN s_articles_details d
                ON (d.ordernumber=b.ordernumber OR g.valueID IS NOT NULL)
                AND d.articleID=a.id
            LEFT JOIN s_articles_attributes at ON at.articledetailsID=d.id
            LEFT JOIN s_core_tax t ON t.id=a.taxID
            LEFT JOIN s_user u ON u.id=NULL
            LEFT JOIN s_user_billingaddress ub ON ub.userID=u.id
            LEFT JOIN s_user_shippingaddress us ON us.userID=u.id
            GROUP BY b.sessionID
        ";
        $basket = Shopware()->Db()->fetchRow(
            $sql,
            array(
                $article["articleID"],
                $article["ordernumber"],
                $article["shippingfree"],
                $article["price"],
                $article["netprice"],
                $article["esd"],
                1,
            )
        )
        ;
        if (empty($basket)) {
            return false;
        }
        
        $basket['countryID']       = $countryID;
        $basket['paymentID']       = $paymentID;
        $basket['customergroupID'] = 1;            // EK (Endkunden)
        $basket['multishopID']     = 1;            // comtech.de
        $basket['sessionID']       = null;
        
        foreach ($basket as $key => $value) {
            if ($value == '') {
                $value = 0;
            }
            $basket[$key] = $value;
        }
        
        return $basket;
    }
    
    protected function getPremiumDispatch($basket, $dispatch = null)
    {
        if (empty($dispatch)) {
            $sql_order = "";
        } elseif (is_numeric($dispatch)) {
            $sql_order = "IF(d.id=" . (int)$dispatch . ",0,1),";
        } elseif (is_string($dispatch)) {
            $sql_order = "IF(d.name=" . Shopware()->Db()->prepare($dispatch) . ",0,1),";
        } else {
            $sql_order = "";
        }
        
        $sql_add_join = "";
        if (!empty($basket['paymentID'])) {
            $sql_add_join .= "
            JOIN s_premium_dispatch_paymentmeans dp
            ON d.id = dp.dispatchID
            AND dp.paymentID={$basket['paymentID']}
            ";
        }
        if (!empty($basket['countryID'])) {
            $sql_add_join .= "
            JOIN s_premium_dispatch_countries dc
            ON d.id = dc.dispatchID
            AND dc.countryID={$basket['countryID']}
            ";
        }
        
        $sql        = "SELECT id, bind_sql FROM s_premium_dispatch WHERE type IN (0) AND bind_sql IS NOT NULL";
        $statements = Shopware()->Db()->fetchAll($sql);
        
        $sql_where = "";
        for ($i = 0; $i < count($statements); $i++) {
            $dispatchID = $statements[$i]['id'];
            $statement  = $statements[$i]['bind_sql'];
            $sql_where .= "
            AND ( d.id!=$dispatchID OR ($statement))
            ";
        }
        
        $sql_basket = array();
        foreach ($basket as $key => $value) {
            if (is_string($value)) {
                $value = "'" . $value . "'";
            }
            $sql_basket[] = $value . " as `$key`";
        }
        $sql_basket = implode(', ', $sql_basket);
        
        $sql      = "
        SELECT d.id, d.name, d.description, d.calculation, d.status_link, d.surcharge_calculation, d.bind_shippingfree, tax_calculation, t.tax as tax_calculation_value, d.shippingfree
        FROM s_premium_dispatch d

        JOIN ( SELECT $sql_basket ) b

        $sql_add_join
        LEFT JOIN (
        SELECT dc.dispatchID
        FROM s_articles_categories ac,
        s_premium_dispatch_categories dc
        WHERE ac.articleID={$basket['articleID']}
        AND dc.categoryID=ac.categoryID
        GROUP BY dc.dispatchID
        ) as dk
        ON d.id = dk.dispatchID

        LEFT JOIN s_core_tax t
        ON t.id=d.tax_calculation

        LEFT JOIN s_user u
        ON u.id=0
        AND u.active=1

        LEFT JOIN s_user_billingaddress ub
        ON ub.userID=u.id

        LEFT JOIN s_user_shippingaddress us
        ON us.userID=u.id

        WHERE d.active = 1
        AND (bind_weight_from IS NULL OR bind_weight_from <= b.weight)
        AND (bind_weight_to IS NULL OR bind_weight_to >= b.weight)
        AND (bind_price_from IS NULL OR bind_price_from <= b.amount)
        AND (bind_price_to IS NULL OR bind_price_to >= b.amount)
        AND (bind_laststock=0 OR (bind_laststock=1 AND b.instock) OR (bind_laststock=2 AND b.stockmin))
        AND (bind_shippingfree!=1 OR NOT b.shippingfree)
        AND (d.multishopID IS NULL OR d.multishopID= b.multishopID)
        AND (d.customergroupID IS NULL OR d.customergroupID=b.customergroupID)
        AND dk.dispatchID IS NULL
        AND d.type IN (0)
        $sql_where
        ORDER BY $sql_order d.position, d.name
        LIMIT 1
        ";
        $dispatch = Shopware()->Db()->fetchRow($sql);
        
        if (empty($dispatch)) {
            $sql      = "
            SELECT
            d.id, d.name,
            d.description,
            d.calculation,
            d.status_link,
            d.surcharge_calculation,
            d.bind_shippingfree,
            tax_calculation,
            t.tax AS tax_calculation_value
            FROM s_premium_dispatch d
            LEFT JOIN s_core_tax t
            ON t.id=d.tax_calculation
            WHERE d.active=1
            AND d.type=1
            ORDER BY d.position, d.name
            LIMIT 1
            ";
            $dispatch = Shopware()->Db()->fetchOne($sql);
        }
        
        foreach ($dispatch as $key => $value) {
            if ($value == '') {
                $value = 0;
            }
            $dispatch[$key] = $value;
        }
        
        return $dispatch;
        
    }
    
    protected function getPremiumDispatchSurcharge($basket)
    {
        if (empty($basket)) {
            return false;
        }
        
        $sql        = 'SELECT id, bind_sql FROM s_premium_dispatch WHERE type=2 AND bind_sql IS NOT NULL';
        $statements = Shopware()->Db()->fetchAll($sql);
        
        $sql_where = '';
        for ($i = 0; $i < count($statements); $i++) {
            $dispatchID = $statements[$i]['id'];
            $statement  = $statements[$i]['bind_sql'];
            $sql_where .= "
            AND ( d.id!=$dispatchID OR ($statement))
            ";
        }
        
        $sql_basket = array();
        foreach ($basket as $key => $value) {
            if (is_string($value)) {
                $value = "'" . $value . "'";
            }
            $sql_basket[] = $value . " as `$key`";
        }
        $sql_basket = implode(', ', $sql_basket);
        
        $sql_add_join = "";
        if (!empty($basket['paymentID'])) {
            $sql_add_join .= "
            JOIN s_premium_dispatch_paymentmeans dp
            ON d.id = dp.dispatchID
            AND dp.paymentID={$basket['paymentID']}
            ";
        }
        if (!empty($basket['countryID'])) {
            $sql_add_join .= "
            JOIN s_premium_dispatch_countries dc
            ON d.id = dc.dispatchID
            AND dc.countryID={$basket['countryID']}
            ";
        }
        
        $sql        = "
        SELECT d.id, d.calculation
        FROM s_premium_dispatch d

        JOIN ( SELECT $sql_basket ) b

        $sql_add_join

        LEFT JOIN (
        SELECT dc.dispatchID
        FROM s_articles_categories ac,
        s_premium_dispatch_categories dc
        WHERE ac.articleID={$basket['articleID']}
        AND dc.categoryID=ac.categoryID
        GROUP BY dc.dispatchID
        ) as dk
        ON dk.dispatchID=d.id

        LEFT JOIN s_user u
        ON u.id=b.userID
        AND u.active=1

        LEFT JOIN s_user_billingaddress ub
        ON ub.userID=u.id

        LEFT JOIN s_user_shippingaddress us
        ON us.userID=u.id

        WHERE d.active=1
        AND (bind_weight_from IS NULL OR bind_weight_from <= b.weight)
        AND (bind_weight_to IS NULL OR bind_weight_to >= b.weight)
        AND (bind_price_from IS NULL OR bind_price_from <= b.amount)
        AND (bind_price_to IS NULL OR bind_price_to >= b.amount)
        AND (bind_instock=0 OR bind_instock IS NULL OR (bind_instock=1 AND b.instock) OR (bind_instock=2 AND b.stockmin))
        AND (bind_laststock=0 OR (bind_laststock=1 AND b.laststock))
        AND (bind_shippingfree=2 OR NOT b.shippingfree)

        AND (d.multishopID IS NULL OR d.multishopID=b.multishopID)
        AND (d.customergroupID IS NULL OR d.customergroupID=b.customergroupID)
        AND dk.dispatchID IS NULL
        AND d.type = 2
        AND (d.shippingfree IS NULL OR d.shippingfree > b.amount)
        $sql_where
        GROUP BY d.id
        ";
        $dispatches = Shopware()->Db()->fetchAll($sql);
        
        $surcharge = 0;
        if (!empty($dispatches)) {
            for ($i = 0; $i < count($dispatches); $i++) {
                $dispatch = $dispatches[$i];
                if (empty($dispatch['calculation'])) {
                    $from = round($basket['weight'], 3);
                } elseif ($dispatch['calculation'] == 1) {
                    $from = round($basket['amount'], 2);
                } elseif ($dispatch['calculation'] == 2) {
                    $from = round($basket['count_article']);
                } elseif ($dispatch['calculation'] == 3) {
                    $from = round($basket['calculation_value_' . $dispatch['id']]);
                } else {
                    continue;
                }
                $sql = "
                    SELECT `value` , `factor`
                    FROM `s_premium_shippingcosts`
                    WHERE `from` <= $from
                    AND `dispatchID` = {$dispatch['id']}
                    ORDER BY `from` DESC
                    LIMIT 1
                ";
                $result = Shopware()->Db()->fetchRow($sql);
                
                if (empty($result)) {
                    continue;
                }
                $surcharge += $result['value'];
                if (!empty($result['factor'])) {
                    $surcharge += $result['factor'] / 100 * $from;
                }
            }
        }
        
        return $surcharge;
        
    }
    
    
    
    ############################################################################
    ## ONE DIMENSIONAL VARIATIONS                                             ##
    ############################################################################
    
    /**
     * generate article with one-dimensional variants
     *
     * @return array
     */
    protected function buildVariationArticles($sgItem, $article)
    {
        $children = array();
        
        /* first child */
        $sgItem['parent_item_number'] = 'sgParent' . $article['articleID'];
        $sgItem['attribute_1']        = $article['additionaltext'];
        
        $parentImages          = explode("||", $sgItem['urls_images']);
        $images                =
            $this->getVariationImages($article['ordernumber'], $article['image'], $article['images']);
        $images                = explode("||", $images);
        $images                = array_merge($parentImages, $images);
        $sgItem['urls_images'] = implode("||", array_unique($images));
        
        $children[] = $sgItem;
        
        /* other children */
        foreach ($article['sVariants'] as $variant) {
            $variant['tax'] = $article['tax'];
            $sgItem         = $this->itemExportItemNumber($sgItem, $variant);
            $sgItem         = $this->itemExportUnitAmount($sgItem, $variant);
            $sgItem         = $this->itemExportOldUnitAmount($sgItem, $variant);
            $sgItem         = $this->itemExportOldUnitAmountNet($sgItem, $variant);
            $sgItem         = $this->itemExportStockQuantity($sgItem, $variant);
            $sgItem         = $this->itemExportAvailableText($sgItem, $variant);
            $sgItem         = $this->itemExportIsFreeShipping($sgItem, $variant);
            
            $sgItem['attribute_1'] = $variant['additionaltext'];
            
            $images                =
                $this->getVariationImages($variant['ordernumber'], $article['image'], $article['images']);
            $images                = explode("||", $images);
            $images                = array_merge($parentImages, $images);
            $sgItem['urls_images'] = implode("||", array_unique($images));
            
            $sgItem['manufacturer_item_number'] = $this->getManufacturerItemNumber($variant['articleDetailsID']);
            
            $children[] = $sgItem;
        }
        
        return $children;
    }
    
    /**
     * get the images of an article with one-dimensional variants
     *
     * @return string
     */
    protected function getVariationImages($item_number, $image, $images = null)
    {
        $imagePaths = array();
        if (empty($image)) {
            return;
        }
        
        $images = array_merge(array($image), $images);
        
        /*** check if images are related to an item_number ***/
        foreach ($images as $image) {
            if ($image['main'] == 1) {
                if ($image['res']['relations'] == $item_number) {
                    $imagePaths[] = $image['src']['original'];
                }
            }
            if ($image['relations'] == $item_number) {
                $imagePaths[] = $image['src']['original'];
            }
        }
        if (empty($imagePaths)) {
            $imagePaths[] = $image['src']['original'];
        }
        
        return implode('||', $imagePaths);
    }
    
    /**
     * Generate the current Item to a Parent-Item
     *
     * Use sgParent<ID> as item_number because Shopware dont use Parent-Child-Relations
     *
     * @param array $sgItem
     * @param array $article
     *
     * @return array
     */
    protected function buildShopgateParentArticle($sgItem, $article)
    {
        $sgItem['has_children'] = 1;
        $sgItem['item_number']  = 'sgParent' . $article['articleID'];
        $sgItem['attribute_1']  = 'Auswahl';
        
        return $sgItem;
    }
    
    
    
    ############################################################################
    ## ARTICLE CONFIGURATOR                                                   ##
    ############################################################################
    
    /*** get the image/images of an configurator-article ***/
    protected function getConfiguratorImages($sgItem, $article)
    {
        if (empty($article)) {
            return;
        }
        
        $imagePaths = array();
        $attributes = array();
        $groups     = $article['sConfigurator'];
        
        $images = $this->export->sArticleImages(array('articleID' => $article['articleID']));
        if (empty($images)) {
            return;
        }
        
        for ($i = 1; !empty($sgItem['attribute_' . $i]); $i++) {
            foreach ($groups as $group) {
                if ($group['groupID'] == $i) {
                    // remove all string elements that can destroy the syntax of the relation-definition
                    $attributes[] = strtolower(
                        str_replace(
                            array(' ', '/', '{', '}'),
                            array('', '', '', ''),
                            str_replace(':', '', $group['groupname'])
                            . ':'
                            . str_replace(':', '', html_entity_decode($sgItem['attribute_' . $i]))
                        )
                    );
                }
            }
        }
        
        foreach ($images[$article['articleID']] as $image) {
            /*** AND ***/
            if (substr($image['relations'], 0, 1) == '&') {
                $valid               = true;
                $cleanRelations      = substr(
                    $image['relations'], 2, /*(strlen($image['relations']) - 3) */
                    -1
                );
                $cleanRelationsArray = explode('/', $cleanRelations);
                
                foreach ($cleanRelationsArray as $cleanRelation) {
                    if (!in_array($cleanRelation, $attributes)) {
                        $valid = false;
                        break;
                    }
                }
            }
            
            /*** OR ***/
            if (substr($image['relations'], 0, 2) == '||') {
                $valid               = false;
                $cleanRelations      = substr(
                    $image['relations'], 3, /*(strlen($image['relations']) - 4)*/
                    -1
                );
                $cleanRelationsArray = explode('/', $cleanRelations);
                
                foreach ($cleanRelationsArray as $cleanRelation) {
                    foreach ($attributes as $attribute) {
                        if ($attribute == $cleanRelation) {
                            $valid = true;
                            break(2);
                        }
                    }
                }
            }
            
            if ($valid) {
                $imagePaths[] = $image['link'];
                $valid        = false;
            }
        }
        
        return $imagePaths;
    }
    
    /*** generate upprice article ***/
    protected function buildUppriceArticle($sgItem, $article)
    {
        /*** has_options = 1, option_1 = name, option_1_values = values ***/
        $sgItem['has_options'] = 1;
        $optionNr              = 1;
        $startingPrice         = $article['priceStartingFrom'];
        
        /*** export of upprice-articles is not recommended -> get data from db ***/
        $options = $this->getUppriceOptions($article['articleID']);
        
        foreach ($options as $option) {
            $sgItem['option_' . $optionNr] = $option['groupID'] . '=' . $option['groupname'];
            $values                        = array();
            $basePrice                     = $article['priceStartingFrom'];
            
            foreach ($option['values'] as $value) {
                $price    = ($value['price'] * (1 + $article['tax'] / 100));
                $values[] = $value['optionID'] . '=' . $value['optionname'] . '=>' . round($price * 100);
            }
            
            $values                                    = implode('||', $values);
            $sgItem['option_' . $optionNr . '_values'] = $values;
            $optionNr++;
        }
        
        $sgItem['unit_amount']    = $this->formatPriceNumber($basePrice);
        $sgItem['stock_quantity'] = $this->getUppriceStock($article['articleID']);
        
        return $sgItem;
    }
    
    /*** get stock of upprice article ***/
    protected function getUppriceStock($articleID)
    {
        $sql    = "
            SELECT
                instock
            FROM s_articles_details
            WHERE articleID = '" . $articleID . "'
        ";
        $result = Shopware()->Db()->fetchOne($sql);
        
        return $result;
    }
    
    /*** get upprice options from database ***/
    protected function getUppriceOptions($articleID)
    {
        $sql = "
            SELECT
                groupID,
                groupname
            FROM s_articles_groups
            WHERE articleID = '" . $articleID . "'
            ORDER BY groupposition
        ";
        $result  = Shopware()->Db()->fetchAll($sql);
        $options = array();
        
        foreach ($result as $option) {
            $option['values'] = $this->getOptionValues($articleID, $option['groupID']);
            $options[]        = $option;
        }
        
        return $options;
    }
    
    /*** get values from given option ***/
    protected function getOptionValues($articleID, $groupID)
    {
        $sql    = "
            SELECT
                sago.optionID,
                sago.optionname,
                sagp.price
            FROM s_articles_groups_option sago
            JOIN s_articles_groups_prices sagp ON sago.optionID = sagp.optionID
            WHERE sago.optionactive = 1
                AND sago.articleID 	= '" . $articleID . "'
                AND sago.groupID 	= '" . $groupID . "'
                AND sagp.groupkey 	= '" . $this->subShop['defaultcustomergroup'] . "'
        ";
        $values = Shopware()->Db()->fetchAll($sql);
        
        return $values;
    }
    
    /*** generate a shopgate parent-article from configurator-article ***/
    protected function buildShopgateConfiguratorParentArticle($sgItem, $article)
    {
        $sgItem['has_children']             = 1;
        $sgItem['item_number']              = 'sgParent' . $article['articleID'];
        $sgItem['stock_quantity']           = 1;
        $sgItem['manufacturer_item_number'] = '';
        
        /*** get groups -> group ID == attribute_x ***/
        $attribute = 1;
        foreach ($article['sConfigurator'] as $group) {
            $sgItem["attribute_{$attribute}"] = $group['groupname'];
            $attribute++;
        }

// 		$imageStack = array();

// 		if (isset($article['image']['src']['original'])){
// 			$imageStack[] = $article['image']['src']['original'];
// 		}

// 		//$images = $this->export->sArticleImages(array('articleID'=>$article['articleID']));
// 		// // $images = array_pop($images);
// 		//$images = empty($images[$article['articleID']]) ? array() : $images[$article['articleID']];
// 		$images = $article['images'];

// 		foreach ($images as $image) {
// 			// Exclude images that are not to be shown for the parent
// 			if (empty($image['relations']) || $image['relations'] == $article['ordernumber']) {
// 				if (!in_array($image["link"], $imageStack))
// 					$imageStack[] = $image["link"];
// 			}
// 		}

// 		$sgItem['urls_images'] = implode("||", $imageStack);
        
        return $sgItem;
    }
    
    /*** generate configurator article ***/
    protected function buildConfiguratorArticles($sgItem, $article)
    {
        $children = array();
        $sql      = "
            SELECT
                agv.*,
                agp.price
            FROM s_articles_groups_value agv
            JOIN s_articles_groups_prices agp
              ON agv.valueID = agp.valueID
            WHERE agv.articleID = '" . $article['articleID'] . "'
              AND agv.active 		= 1
              AND agp.groupkey 	= '" . $this->subShop['defaultcustomergroup'] . "'
        ";
        
        $result = Shopware()->Db()->query($sql);
        
        while ($row = $result->fetch()) {
            $attribute = 1;
            for ($i = 1; $i <= 10; $i++) {
                if ($row["attr{$i}"]) {
                    $sgItem["attribute_{$attribute}"] = htmlentities($this->getOptionName($row["attr{$i}"]));
                    $attribute++;
                }
            }
            
            $sgItem['has_children']       = 0;
            $sgItem['parent_item_number'] = 'sgParent' . $article['articleID'];
            $sgItem['item_number_public'] = $row['ordernumber'];
            $sgItem['item_number']        = $row['ordernumber'];
            $sgItem['unit_amount']        = $this->formatPriceNumber($row['price'] * (1 + $article['tax'] / 100), 2);
            
            $sgItem['urls_images'] = implode('||', $this->getConfiguratorImages($sgItem, $article));
            
            $row["shippingtime"] = $article["shippingtime"];
            $sgItem              = $this->itemExportAvailableText($sgItem, $row);
            
            /*** stock ***/
            $sgItem['stock_quantity'] = $row['instock'];
            
            $children[] = $sgItem;
        }
        
        return $children;
    }
    
    /**
     * Findet zur OptionsId den Namen
     *
     * @param $optionId - Id der Option
     *
     * @return string $optionname - Optionsname
     */
    protected function getOptionName($optionId)
    {
        $sql    = "
            SELECT
                sago.optionname
            FROM s_articles_groups_option sago
            WHERE sago.optionID = '" . $optionId . "'
            AND sago.optionactive = 1
        ";
        $result = Shopware()->Db()->fetchOne($sql);
        
        return $result;
    }
    
    
    
    ############################################################################
    ## CREATE ITEMS-CSV HELPER                                                ##
    ############################################################################
    
    /**
     *
     */
    protected function getActiveArticleSqlSnippet()
    {
        $sql = "a.`active` = 1 AND a.`mode` = 0";
        
        return $sql;
    }
    
    protected function buildOrderArticleName($name, $options)
    {
        $returnString = $name;
        $strings      = array();
        
        foreach ($options as $option) {
            $strings[] = "{$option->getName()}: {$option->getValue()}";
        }
        
        $returnString .= " - " . implode(" / ", $strings);
        
        return $returnString;
    }
    
    protected function getShopgatePaymentID()
    {
        $sql    = "
            SELECT
                id
            FROM s_core_paymentmeans
            WHERE name = 'shopgate'
        ";
        $result = Shopware()->Db()->fetchOne($sql);
        
        return $result;
    }
    
    protected function getTaxIdByAmount($amount)
    {
        $sql    = "
            SELECT
                id
            FROM s_core_tax
            WHERE tax = '" . $amount . "'
        ";
        $result = Shopware()->Db()->fetchOne($sql);
        
        return $result;
    }
    
    protected function _buildOrderArticleName($name, $attributes, $options)
    {
        $returnString = $name;
        foreach ($attributes as $attribute) {
            $returnString .= ' ' . $attribute->getName() . ':';
            $returnString .= ' ' . $attribute->getValue();
        }
        foreach ($options as $option) {
            $returnString .= ' ' . $option->getName() . ':';
            $returnString .= ' ' . $option->getValue();
        }
        
        return    //utf8_decode( // $string should already be decoded from UTF-8 to ISO-8859-15
            $returnString//);
            ;
    }
    
    /**
     *
     * @param string $iso
     *
     * @return integer
     */
    protected function getCountryByIso($iso)
    {
        $sql     = "SELECT id FROM s_core_countries WHERE countryiso = '{$iso}'";
        $country = Shopware()->Db()->fetchOne($sql);
        
        return $country;
    }
    
    /**
     * get article details by articleID
     *
     * @param int $id -> id of the article
     *
     * @return array -> articledata
     */
    protected function getArticleById($articleId)
    {
        $sql = "
            SELECT
                aDetails.id as articleDetailsID,
                TRIM(aDetails.ordernumber) as ordernumber,
                aDetails.additionaltext,
                aDetails.sales,
                aDetails.instock,
    
                a.id as articleID,
                a.datum,
                a.shippingtime,
                a.shippingfree,
                a.topseller,
                a.notification,
                a.purchasesteps,
                a.maxpurchase,
                a.minpurchase,
                a.purchaseunit,
                a.referenceunit,
                a.packunit,
                aDetails.weight,
                a.laststock,
                a.unitID,
                a.template,
                a.pricegroupID,
                a.pricegroupActive,
                a.releasedate,
                a.mode,
                a.description,
                a.keywords,
                a.description_long,
                a.name AS articleName,
                a.filtergroupID,
                a.crossbundlelook,
                aDetails.esd as esd,
    
                aSupplier.name AS supplierName,
                aSupplier.img AS supplierImg,
                aSupplier.id AS supplierID,
    
                IFNULL(p.price,p2.price) as price,
                IF(p.pseudoprice,p.pseudoprice,p2.pseudoprice) as pseudoprice,
                IFNULL(p.pricegroup,p2.pricegroup) as pricegroup,
    
                aTax.id AS taxID,
                aTax.tax,
                attr1,attr2,attr3,attr4,attr5,attr6,attr7,attr8,attr9,attr10,
                attr11,attr12,attr13,attr14,attr15,attr16,attr17,attr18,attr19,attr20
                    
            FROM s_articles AS a
            LEFT JOIN s_articles_supplier AS aSupplier
                ON ( a.supplierID = aSupplier.id )
            LEFT JOIN s_articles_details AS aDetails
                ON ( a.id = aDetails.articleID AND aDetails.kind = 1 )
            LEFT JOIN s_articles_prices AS p
                ON ( p.articledetailsID = aDetails.id
                AND  p.pricegroup='" . $this->sSYSTEM->sUSERGROUP . "'
                AND  p.from='1' )
            LEFT JOIN s_articles_prices AS p2
                ON ( p2.articledetailsID=aDetails.id
                AND  p2.pricegroup='EK'
                AND  p2.from='1' )
            LEFT JOIN s_core_tax AS aTax
                ON ( aTax.id = a.taxID )
            LEFT JOIN s_articles_attributes AS aAttributes
                ON ( aAttributes.articleID = a.id AND aAttributes.articledetailsID = aDetails.id )
            WHERE  a.id = {$articleId}
        ";
        
        $getArticle = Shopware()->Db()->fetchRow($sql);
        if (empty($getArticle)) {
            return 0;
        }
        
        $laststockConfig    = "
            SELECT
                IF(ags.instock IS NULL, 0, ags.instock) AS laststock
            FROM `s_articles_groups_value` AS agv
            LEFT JOIN `s_articles_groups_settings` AS ags ON(ags.articleID = agv.articleID)
            WHERE agv.`articleID` = '" . $getArticle['articleID'] . "'
            LIMIT 1
        ";
        $getLaststockConfig = Shopware()->Db()->fetchRow($laststockConfig);
        if (!empty($getLaststockConfig)) {
            $getArticle['laststock'] = $getLaststockConfig['laststock'];
        }
        
        /*** Translation ***/
        $getArticle = Shopware()->Modules()->Articles()->sGetTranslation(
            $getArticle, $getArticle['articleID'], "article", $this->sSYSTEM->sLanguage
        )
        ;
        
        /*** article variants (one-dimensional) ***/
        $getArticle['sVariants'] = $this->getArticleVariants(
            $getArticle['articleID'], $getArticle['tax'], $getArticle['shippingtime'], $getArticle['shippingfree']
        );
        /*** configurator article ***/
        $getArticle["sConfigurator"]         = $this->getArticleConfig($getArticle["articleID"], $getArticle);
        $getArticle["sConfiguratorSettings"] = $this->getArticleConfigSettings($getArticle["articleID"], $getArticle);
// 		$getArticle 						= $this->getArticleConfig($getArticle["articleID"],$getArticle);
        
        /*** images ***/
        $getArticle["image"]  = $this->getArticlePictures($getArticle["articleID"], true, 4); //ändern
        $getArticle["images"] = $this->getArticlePictures($getArticle["articleID"], false, 0); //ändern
        
        /*** deeplink ***/
        $getArticle["linkDetails"] =
            $this->sSYSTEM->sCONFIG['sBASEFILE'] . "?sViewport=detail&sArticle=" . $getArticle["articleID"];
        $getArticle["linkDetailsRewrited"] =
            $this->sSYSTEM->sMODULES['sCore']->sRewriteLink($getArticle["linkDetails"], $getArticle["articleName"]);
        /*** price and starting price ***/
        
        $getArticle["price"]             = $this->buildPrice($getArticle['price'], $getArticle['tax']);
        $getArticle["priceStartingFrom"] = $this->getCheapestPrice($getArticle["articleID"], $getArticle["tax"]);
        
        /*** article relations ***/
        $getArticle["sRelatedArticles"] = $this->getRelatedArticles($getArticle["articleID"]);
        $getArticle["sSimilarArticles"] = $this->getSimilarArticles($getArticle["articleID"]);
        
        return $getArticle;
    }
    
    /**
     * fetch accessories by articleID
     *
     * @param $articleId
     *
     * @return array
     */
    protected function getRelatedArticles($articleId)
    {
        $result = array();
        
        $getRelatedArticles = Shopware()->Db()->fetchAll(
            "SELECT relatedarticle FROM s_articles_relationships WHERE articleID={$articleId}"
        );
        
        foreach ($getRelatedArticles as $relatedArticleValue) {
            $result[] = $relatedArticleValue;
        }
        
        return $result;
    }
    
    /**
     * fetch similar articles by articleID
     *
     * @param $articleId
     *
     * @return array
     */
    protected function getSimilarArticles($articleId)
    {
        $result = array();
        
        $getSimilarArticles = Shopware()->Db()->fetchAll(
            "SELECT relatedarticle FROM s_articles_similar WHERE articleID={$articleId}"
        );
        
        foreach ($getSimilarArticles as $getSimilarArticle) {
            $result[] = $getSimilarArticle;
        }
        
        return $result;
    }
    
    /**
     * get price with tax
     *
     */
    protected function buildPrice($price, $tax)
    {
        $price = (float)$price;
        $tax   = (float)$tax;
        
        if ($this->sSYSTEM->sCurrency['factor']) {
            $price = $price * floatval($this->sSYSTEM->sCurrency["factor"]);
        }
        
        $price = round($price * (100 + $tax) / 100, 3);
        
        return $price;
        
    }
    
    /*** get cheapest price (configurator) ***/
    protected function getCheapestPrice($articleID, $tax)
    {
        $sql    = "
            SELECT price FROM s_articles_prices, s_articles_details WHERE
            s_articles_details.id=s_articles_prices.articledetailsID AND
            pricegroup='EK'
            AND s_articles_details.articleID='" . $articleID . "'
            GROUP BY ROUND(price,2)
            ORDER BY price ASC
            LIMIT 2
        ";
        $result = Shopware()->Db()->fetchAll($sql);
        if (!empty($result)) {
            $price = $result[0]['price'];
            
            return round($price * (100 + $tax) / 100, 3);
        } else {
            return 0;
        }
        
    }
    
    /*** get one dimensional variants ***/
    protected function getArticleVariants($articleID, $tax, $shippingtime, $shippingfree)
    {
        $sql                = "
        SELECT s_articles_details.id, s_articles_details.id AS articleDetailsID, ordernumber, suppliernumber, weight,additionaltext, price, pseudoprice,pricegroup,
        instock,
        attr1, attr2, attr3, attr4, attr5, attr6, attr7, attr8, attr9, attr10, attr11, attr12,attr13,attr14,attr15,attr16,attr17,attr19,attr20
        FROM s_articles_details
        LEFT JOIN s_articles_prices ON s_articles_details.id=s_articles_prices.articledetailsID
        AND ( s_articles_prices.pricegroup= '" . $this->sSYSTEM->sUSERGROUP . "' )
        , s_articles_attributes
        WHERE
        s_articles_details.articleID='" . $articleID . "' AND
        s_articles_attributes.articledetailsID=s_articles_details.id
        AND s_articles_details.kind=2
        GROUP BY articleDetailsID
        ORDER BY s_articles_details.kind ASC, s_articles_details.position ASC
        ";
        $getArticleVariants = Shopware()->Db()->fetchAll($sql);
        
        $getArticle = Shopware()->Modules()->Articles()->sGetArticleById($articleID);
        
        foreach ($getArticleVariants as $variantKey => $variantValue) {
            if (!empty($this->sSYSTEM->sCONFIG["sLIVEINSTOCK"])) {
                $getArticleVariants[$variantKey]["instock"] = $this->sSYSTEM->sDB_CONNECTION->GetOne(
                    "
                        SELECT instock FROM s_articles_details WHERE id = {$variantValue["articleDetailsID"]}
                "
                );
            }
            if (empty($getArticleVariants[$variantKey]["shippingtime"])) {
                // Fix the bug, that shippingtime is only available for the first article
                $getArticleVariants[$variantKey]["shippingtime"] = $shippingtime;
            }
            if (empty($getArticleVariants[$variantKey]["shippingfree"])) {
                // Fix the bug, that shippingtime is only available for the first article
                $getArticleVariants[$variantKey]["shippingfree"] = $shippingfree;
            }
            list($Y, $M, $D) = explode("-", $getArticle["releasedate"]);
            if (mktime(0, 0, 0, $M, $D, $Y) > mktime(0, 0, 0, date("m"), date("d"), date("Y"))) {
                $getArticleVariants[$variantKey]["sReleaseDate"] = $D . "." . $M . "." . $Y;
            }
            $getArticleVariants[$variantKey]['price'] =
                $this->buildPrice($getArticleVariants[$variantKey]['price'], $tax);
        }
        
        if (!empty($getArticleVariants)) {
            return $getArticleVariants;
        } else {
            return array();
        }
    }
    
    protected function getArticleConfigSettings($id, $article)
    {
        $sql = "
            SELECT
                *
            FROM s_articles_groups_settings ags
            WHERE ags.articleID = {$id}";
        
        $result = Shopware()->Db()->fetchRow($sql);
        
        return $result;
    }
    
    /*** get configurator data ***/
    protected function getArticleConfig($id, $article)
    {
        $sql = "
            SELECT
                ag.*
            FROM s_articles_groups ag
            WHERE ag.articleID = {$id}
            ORDER BY ag.groupID
        ";
        
        $result = Shopware()->Db()->query($sql);
        $groups = array();
        while ($row = $result->fetch()) {
            $groups[] = $row;
        }
        
        
        return $groups;
    }
    
    /*** get pictures data ***/
    protected function getArticlePictures(
        $sArticleID, $onlyCover = true, $pictureSize, $ordernumber = "", $allImages = false, $realtime = false
    ) {
        $sArticleID = intval($sArticleID);
        if (empty($this->sSYSTEM->sPathArticleImg)) {
            if (preg_match("/443/", $_SERVER['SERVER_PORT'])) {
                $this->sSYSTEM->sPathArticleImg =
                    "https://" . $this->sSYSTEM->sCONFIG["sBASEPATH"] . $this->sSYSTEM->sCONFIG["sARTICLEIMAGES"] . "/";
            } else {
                $this->sSYSTEM->sPathArticleImg =
                    "http://" . $this->sSYSTEM->sCONFIG["sBASEPATH"] . $this->sSYSTEM->sCONFIG["sARTICLEIMAGES"] . "/";
            }
        }
        
        $imagesizes = $this->sSYSTEM->sCONFIG['sIMAGESIZES'];
        
        $imagesizes = explode(";", $imagesizes);
        foreach ($imagesizes as $imagesize) {
            $imagesize = explode(":", $imagesize);
            $sizes[]   = $imagesize[2];
        }
        
        /*** main picture ***/
        if ($onlyCover) {
            if (empty($ordernumber)) {
                $query_image = $this->sSYSTEM->sDB_CONNECTION->CacheGetRow(
                    $realtime == true ? 0 : $this->sSYSTEM->sCONFIG['sCACHEARTICLE'], "
                        SELECT * FROM s_articles_img WHERE articleID=? ORDER BY main LIMIT 1
                        ", array($sArticleID), "article_$sArticleID"
                );
            } else {
                $query_image = $this->sSYSTEM->sDB_CONNECTION->CacheGetRow(
                    $realtime == true ? 0 : $this->sSYSTEM->sCONFIG['sCACHEARTICLE'], "
                        (
                        SELECT * FROM s_articles_img WHERE articleID=$sArticleID AND relations LIKE '%$ordernumber%' LIMIT 1
                ) UNION ALL (
                        SELECT * FROM s_articles_img WHERE articleID=$sArticleID AND main=1
                ) LIMIT 1
                        ", false, "article_$sArticleID"
                );
            }
            
            if (!empty($query_image["img"])) {
                
                if (empty($query_image["extension"])) {
                    $query_image["extension"] = "jpg";
                }
                $result["src"]["original"]           =
                    $this->sSYSTEM->sPathArticleImg . $query_image["img"] . "." . $query_image["extension"];
                $result["res"]["original"]["width"]  = $query_image["width"];
                $result["res"]["original"]["height"] = $query_image["height"];
                $result["res"]["description"]        = $query_image["description"];
                $result["res"]["relations"]          = $query_image["relations"];
                $result["position"]                  = $query_image["position"];
                $result["extension"]                 = $query_image["extension"];
                $result["main"]                      = $query_image["main"];
                foreach ($sizes as $size) {
                    $result["src"][$size] =
                        $this->sSYSTEM->sPathArticleImg . $query_image["img"] . "_$size." . $query_image["extension"];
                }
                
            } else {
                foreach ($sizes as $size) {
                    //$result["src"][$size] = "";
                }
                $result["resolution"] = array();
            }
        } else {
            /*** all article pictures ***/
            if (empty($allImages)) {
                $query_image = $this->sSYSTEM->sDB_CONNECTION->CacheGetAll(
                    $realtime == true ? 0 : $this->sSYSTEM->sCONFIG['sCACHEARTICLE'], "
                        SELECT * FROM s_articles_img WHERE articleID=? AND main=2 ORDER BY position, id ASC
                        ", array($sArticleID), "article_$sArticleID"
                );
            } else {
                $query_image = $this->sSYSTEM->sDB_CONNECTION->CacheGetAll(
                    $realtime == true ? 0 : $this->sSYSTEM->sCONFIG['sCACHEARTICLE'], "
                        SELECT * FROM s_articles_img WHERE articleID=? ORDER BY position, id ASC
                        ", array($sArticleID), "article_$sArticleID"
                );
            }
            
            $result = array();
            foreach ($query_image as $imageKey => $image) {
                if (empty($image["extension"])) {
                    $image["extension"] = "jpg";
                }
                $result[$imageKey]["src"]["original"]           =
                    $this->sSYSTEM->sPathArticleImg . $image["img"] . "." . $image["extension"];
                $result[$imageKey]["res"]["original"]["width"]  = $image["width"];
                $result[$imageKey]["res"]["original"]["height"] = $image["height"];
                $result[$imageKey]["description"]               = $image["description"];
                $result[$imageKey]["relations"]                 = $image["relations"];
                $result[$imageKey]["extension"]                 = $image["extension"];
                $result[$imageKey]["position"]                  = $image["position"];
                $result[$imageKey]["main"]                      = $image["main"];
                // Access to every available size
                foreach ($sizes as $size) {
                    $result[$imageKey]["src"][$size] =
                        $this->sSYSTEM->sPathArticleImg . $image["img"] . "_$size." . $image["extension"];
                }
                
            }
        }
        
        $result = Enlight()->Events()->filter(
            'Shopware_Modules_Articles_GetArticlePictures_FilterResult', $result,
            array('subject' => $this, 'id' => $sArticleID)
        )
        ;
        
        return $result;
    }
    
    protected function checkIfConfig($id)
    {
        $sql    = "
        SELECT
        groupID,
        groupname
        FROM s_articles_groups
        WHERE articleId = '" . $id . "'
        ";
        $result = Shopware()->Db()->fetchAll($sql);
        
        if (empty($result)) {
            return false;
        } else {
            return true;
        }
    }
    
    protected function checkIfVariant($id)
    {
        $sql    = "
        SELECT
        articleID
        FROM s_articles_details
        WHERE articleID = '" . $id . "'
        AND kind = 2
        ";
        $result = Shopware()->Db()->fetchAll($sql);
        
        if (empty($result)) {
            return false;
        } else {
            return true;
        }
    }
    
    
    ############################################################################
    ## CART / COUPONS                                                         ##
    ############################################################################
    
    public function redeemCoupons(ShopgateCart $cart)
    {
        // dont need it yet
        
        $coupons = array();
        foreach ($cart->getExternalCoupons() as $coupon) {
            $info = $this->jsonDecode($coupon->getInternalInfo(), true);
            
            // Set code as cashed if given
            if ($info["code_id"]) {
                $sql  = "SELECT * FROM s_emarketing_voucher_codes WHERE id = ?";
                $code = Shopware()->Db()->fetchRow($sql, array($info["code_id"]));
                
                if ($code["cashed"]) {
                    throw new ShopgateLibraryException(ShopgateLibraryException::COUPON_CODE_NOT_VALID);
                }
                
                Shopware()->Db()->update("s_emarketing_voucher_codes", array("cashed" => 1), "id = {$info["code_id"]}");
            }
            
            $coupon->setIsValid(true);
            $coupons[] = $coupon;
        }
        
        return $coupons;
    }
    
    /**
     * Check cart in shopware
     *
     * basket will delete after check and shopgate coupons
     * are ignored on external coupon calculation
     *
     * (non-PHPdoc)
     *
     * @see ShopgatePlugin::checkCart()
     */
    public function checkCart(ShopgateCart $cart)
    {
        if (count($cart->getExternalCoupons()) > 1) {
            throw new ShopgateLibraryException(ShopgateLibraryException::COUPON_TOO_MANY_COUPONS);
        }
        
        $session = Shopware()->Session();
        
        $basket = Shopware()->Modules()->Basket();
        $result = array("external_coupons" => array());
        
        if ($cart->getExternalCustomerId()) {
            $userId = $cart->getExternalCustomerId();
        } else {
            $userId = Shopware()->Db()->fetchOne("SELECT id FROM s_user WHERE email = ?", array($cart->getMail()));
        }
        
        if ($userId) {
            $this->sSYSTEM->_SESSION["sUserId"] = $userId;
        }
        
        // Add all products to basket
        foreach ($cart->getItems() as $item) {
            $info = $this->jsonDecode($item->getInternalOrderInfo(), true);
            
            $articleResult = $basket->sAddArticle(
                $item->getItemNumber(), $item->getQuantity()
            );
        }
        
        // Add external coupons
        foreach ($cart->getExternalCoupons() as $_coupon) {
            $coupon = new ShopgateExternalCoupon();
            $coupon->setCode($_coupon->getCode());
            
            $voucherResult = $basket->sAddVoucher($_coupon->getCode());
            
            if ($voucherResult === null) {
                
                $result = $this->loadVoucher($coupon->getCode());
                
                $voucher = $result["voucher"];
                $code    = $result["code"];
                
                $info                = array();
                $info["voucher_id"]  = $voucher["id"];
                $info["code_id"]     = $code ? $code["id"] : null;
                $info["order_code"]  = $voucher["ordercode"];
                $info["tax_id"]      = $voucher["taxconfig"];
                $info["tax_percent"] = $basket->getMaxTax();
                $info["amount"]      = abs($this->getBasketDiscountAmount($voucher["ordercode"]));
                
                $coupon->setIsValid(true);
                $coupon->setCode($code ? $code["code"] : $voucher["vouchercode"]);
                $coupon->setName($voucher["description"]);
                $coupon->setDescription($voucher["description"]);
                $coupon->setIsFreeShipping($voucher["shippingfree"]);
                $coupon->setAmount($info["amount"]);
                $coupon->setCurrency($this->sSYSTEM->sCurrency['currency']);
                $coupon->setInternalInfo($this->jsonEncode($info));
                
            } else {
                
                $coupon->setIsValid(false);
                
                if (is_array($voucherResult)) {
                    if (isset($voucherResult["sErrorMessages"])) {
                        $message = implode("; ", $voucherResult["sErrorMessages"]);
                        
                        $coupon->setNotValidMessage($message);
                        
                        $coupon = $coupon->utf8Encode(); // need this!
                    }
                }
                
            }
            
            $result["external_coupons"][] = $coupon;
            
        }
        
        // Delete basket
        $basket->sDeleteBasket();
        
        return $result;
    }
    
    /**
     *
     * @param string $code
     *
     * @return \Shopware\Models\Voucher\Voucher
     */
    protected function loadVoucher($code)
    {
        $result = array(
            "voucher" => null,
            "code"    => null
        );
        
        $sql               = "SELECT * FROM s_emarketing_vouchers WHERE vouchercode = ?";
        $result["voucher"] = Shopware()->Db()->fetchRow($sql, array($code));
        
        if (!$result["voucher"]) {
            $sql            = "SELECT * FROM s_emarketing_voucher_codes WHERE code = ?";
            $result["code"] = Shopware()->Db()->fetchRow($sql, array($code));
            
            if ($result["code"]) {
                $sql               = "SELECT * FROM s_emarketing_vouchers WHERE id = ?";
                $result["voucher"] = Shopware()->Db()->fetchRow($sql, array($result["code"]["voucherID"]));
            }
        }
        
        return $result;
    }
    
    protected function getBasketDiscountAmount($voucherOrderCode)
    {
        $result = $this->sSYSTEM->sDB_CONNECTION->GetRow(
            "SELECT
                    SUM(quantity*(floor(price * 100 + .55)/100)) AS totalAmount
                FROM s_order_basket
                WHERE sessionID=?
                AND ordernumber=?
                GROUP BY sessionID",
            array(
                $this->sSYSTEM->sSESSION_ID,
                $voucherOrderCode
            )
        );
        
        return $result["totalAmount"];
    }
    
    ############################################################################
    ## REVIEWS-CSV                                                            ##
    ############################################################################
    
    /**
     * @see lib/ShopgatePluginCore::startCreateReviewsCsv()
     */
    protected function createReviewsCsv()
    {
        $sql = "
            SELECT
                sav.*,
                sad.ordernumber,
                sags.type
            FROM s_articles_vote sav
            LEFT JOIN s_articles_details sad
            ON sav.articleID = sad.articleID
            LEFT JOIN s_articles_groups_settings sags
            ON sav.articleID = sags.articleID
            WHERE sav.active = 1
        ";
        
        if ($this->splittedExport) {
            $sql .= " LIMIT {$this->exportOffset}, {$this->exportLimit}";
        }
        
        $result = Shopware()->Db()->query($sql);
        
        while ($row = $result->fetch()) {
            $itemArr = array();
            
            $itemArr = $this->executeLoaders($this->getCreateReviewsCsvLoaders(), $itemArr, $row);
            
            $this->addReviewRow($itemArr);
        }
    }
    
    protected function reviewExportItemNumber($review, $data)
    {
        /*** check if normal or variant/configurator article or upprice configurator article ***/
        $conf = $this->checkIfConfig($data['articleID']);
        $var  = $this->checkIfVariant($data['articleID']);
        if (!empty($data['type']) && $data['type'] == 3) {
            $upprice = true;
        } else {
            $upprice = false;
        }
        
        if ($var || ($conf && $upprice == false)) {
            $review['item_number'] = 'sgParent' . $data['articleID'];
        } else {
            $review['item_number'] = $data['ordernumber'];
        }
        
        return $review;
    }
    
    protected function reviewExportUpdateReviewId($review, $data)
    {
        $review['update_review_id'] = $data['id'];
        
        return $review;
    }
    
    protected function reviewExportScore($review, $data)
    {
        $review['score'] = $data['points'] * 2;
        
        return $review;
    }
    
    protected function reviewExportName($review, $data)
    {
        $review['name'] = $data['name'];
        
        return $review;
    }
    
    protected function reviewExportTitle($review, $data)
    {
        $review['title'] = $data['headline'];
        
        return $review;
    }
    
    protected function reviewExportText($review, $data)
    {
        $review['text'] = $data['comment'];
        
        return $review;
    }
    
    protected function reviewExportDate($review, $data)
    {
        $review['date'] = $data['datum'];
        
        return $review;
    }
    
    ############################################################################
    ## PAGES-CSV                                                              ##
    ############################################################################
    
    /**
     * NOT IMPLEMENTED
     */
    protected function createPagesCsv()
    {
        
    }
    
    ############################################################################
    ## ORDER                                                                  ##
    ############################################################################
    
    /**
     * (non-PHPdoc)
     *
     * @see ShopgatePlugin::addOrder()
     */
    public function addOrder(ShopgateOrder $oShopgateOrder)
    {
        Shopware()->Db()->beginTransaction();
        
        $this->log(
            "Start to add order into system #{$oShopgateOrder->getOrderNumber()}", ShopgateLogger::LOGTYPE_DEBUG
        );
        
        $this->shopgateOrderData = $this->getSavedShopgateOrder($oShopgateOrder);
        if (!empty($this->shopgateOrderData)) {
            if ($this->shopgateOrderData['order_import_complete'] == '1') {
                $this->log("Duplicate order #{$oShopgateOrder->getOrderNumber()}", ShopgateLogger::LOGTYPE_DEBUG);
                throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_DUPLICATE_ORDER);
            } else {
                $this->log(
                    "Unfinished order import detected #{$oShopgateOrder->getOrderNumber()} (shopware orderID #{$this->shopgateOrderData['orderID']}",
                    ShopgateLogger::LOGTYPE_DEBUG
                );
            }
            
            // try to find an order that could possibly hav not been connected via the orderID field (since it is updated after insertion of the actual order) - but is most unlikely -> only on manual insertion of shopgate order data afterwards or on old data
            if (empty($this->shopgateOrderData['orderID'])) {
                $tmpOrder = Shopware()->Db()->fetchRow(
                    "SELECT id AS `orderID` from `s_order` WHERE SUBSTRING(`transactionID`, 1, 10) = {$oShopgateOrder->getOrderNumber()}"
                )
                ;
                if (!empty($tmpOrder) && !empty($tmpOrder['orderID'])) {
                    $this->shopgateOrderData['orderID'] = $tmpOrder['orderID'];
                }
                unset($tmpOrder);
            }
        }
        
        $this->log("Calling add order loaders...", ShopgateLogger::LOGTYPE_DEBUG);
        /* @var $oShopwareOrder Shopware_Plugins_Backend_SgateShopgatePlugin_Components_SaveOrder */
        $oShopwareOrder = new Shopware_Plugins_Backend_SgateShopgatePlugin_Components_SaveOrder(
            Shopware()->Modules()->Order(), $this->config, $oShopgateOrder->getOrderNumber()
        );
        $oShopwareOrder = $this->executeLoaders($this->getAddOrderLoaders(), $oShopwareOrder, $oShopgateOrder);
        $this->log(
            "Saving order to database Shopgate-Order-Number #{$oShopgateOrder->getOrderNumber()}",
            ShopgateLogger::LOGTYPE_DEBUG
        );
        $this->log("-> Member-data in order object: {", ShopgateLogger::LOGTYPE_DEBUG);
        foreach (get_object_vars($oShopwareOrder) as $key => $value) {
            if (!is_object($value)) {
                $this->log(
                    "\t{$key} = " . str_replace(
                        array("\r", "\n"), array("", "\n\t"),
                        (is_null($value) ? 'NULL' : (empty($value) ? 'EMPTY' : print_r($value, true)))
                    ), ShopgateLogger::LOGTYPE_DEBUG
                );
            } else {
                $this->log(
                    "\t{$key} = [object of type \"" . get_class($value) . "\"] ***see members below***",
                    ShopgateLogger::LOGTYPE_DEBUG
                );
                foreach (get_object_vars($value) as $key2 => $value2) {
                    if (!is_object($value2)) {
                        $this->log(
                            "\t\t{$key2} = " . str_replace(
                                array("\r", "\n"), array("", "\n\t\t"),
                                (is_null($value2) ? 'NULL' : (empty($value2) ? 'EMPTY' : print_r($value2, true)))
                            ), ShopgateLogger::LOGTYPE_DEBUG
                        );
                    } else {
                        $this->log(
                            "\t\t{$key2} = [object of type \"" . get_class($value2) . "\"]",
                            ShopgateLogger::LOGTYPE_DEBUG
                        );
                        $this->log(
                            "\t\t\t***no members of class \"" . get_class($value2) . "\" exposed here***",
                            ShopgateLogger::LOGTYPE_DEBUG
                        );
                    }
                }
            }
        }
        $this->log("-> }", ShopgateLogger::LOGTYPE_DEBUG);
        $iOrderNumber = $oShopwareOrder->sSaveOrder(
            !empty($this->shopgateOrderData['orderID']) ? $this->shopgateOrderData['orderID'] : null
        );
        if (empty($iOrderNumber)) {
            // An error has happened at the last order saving. The order should still have an order number reserved
            $iOrderNumber = $oShopwareOrder->sOrderNumber;
        }
        
        $sql            = "SELECT * FROM s_order WHERE ordernumber = '{$iOrderNumber}'";
        $aShopwareOrder = Shopware()->Db()->fetchAssoc($sql);
        // Skip the surrounding array (id as index), by just taking the first (and only) array element
        if (!empty($aShopwareOrder)) {
            $firstElementsKey = key($aShopwareOrder);
            $aShopwareOrder   = $aShopwareOrder[$firstElementsKey];
        } else {
            // the order was not saved properly, so roll back the db-transaction
            Shopware()->Db()->rollBack();
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_ORDER_NOT_FOUND,
                "The order that has just been saved could not be loaded for the shopgate order number #{$oShopgateOrder->getOrderNumber()}"
            );
        }

// 		$aCustomerShipping = Shopware()->Db()->fetchAssoc("SELECT * FROM s_user_shippingaddress WHERE userID = ".$aShopwareOrder['userID']);
// 		// Skip the surrounding array (id as index), by just taking the first (and only) array element
// 		if (!empty($aCustomerShipping)) {
// 			$firstElementsKey = key($aCustomerShipping);
// 			$aCustomerShipping = $aCustomerShipping[$firstElementsKey];
// 		}

// 		$aOrderShipping = $aCustomerShipping;
        
        $this->log("Calling after add order loaders...", ShopgateLogger::LOGTYPE_DEBUG);
        $this->executeLoaders($this->getAfterAddOrderLoaders(), $aShopwareOrder, $oShopgateOrder);
        
        // mark the order as "order-data-import complete"
        $this->markShopgateOrderImportComplete($oShopgateOrder->getOrderNumber());
        
        $this->log("Finish add order!", ShopgateLogger::LOGTYPE_DEBUG);
        
        Shopware()->Db()->commit();
        
        $data = array(
            "external_order_id"     => $aShopwareOrder['id'],
            "external_order_number" => $iOrderNumber
        );
        
        return $data;
    }
    
    /**
     * (non-PHPdoc)
     *
     * @see ShopgatePlugin::updateOrder()
     */
    public function updateOrder(ShopgateOrder $oShopgateOrder)
    {
        Shopware()->Db()->beginTransaction();
        
        $iOrder = $this->getSavedShopgateOrder($oShopgateOrder);
        if (empty($iOrder['orderID'])) {
            throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_ORDER_NOT_FOUND);
        }
        
        $oShopgateOrder = $this->prepareShopgateOrder($oShopgateOrder);
        
        $sql    = "SELECT * FROM `s_order` WHERE id = {$iOrder['orderID']}";
        $aOrder = Shopware()->Db()->fetchAssoc($sql);
        $aOrder = reset($aOrder);
        
        if ($oShopgateOrder->getUpdatePayment()) {
            $aOrder = $this->setOrderPayment($aOrder, $oShopgateOrder);
            $aOrder = $this->setOrderClear($aOrder, $oShopgateOrder);
        }
        if ($oShopgateOrder->getUpdateShipping()) {
            $aOrder = $this->setOrderStatus($aOrder, $oShopgateOrder);
        }
        
        Shopware()->Db()->commit();
        
        return array(
            "external_order_id"     => $iOrder['orderID'],
            "external_order_number" => $aOrder["ordernumber"]
        );
    }
    
    protected function getSavedShopgateOrder(ShopgateOrder $oShopgateOrder)
    {
        $sql    = "
            SELECT *
            FROM s_shopgate_orders
            WHERE shopgate_order_number = '{$oShopgateOrder->getOrderNumber()}'
        ";
        $result = Shopware()->Db()->fetchRow($sql);
        
        return $result;
    }
    
    protected function createUser(ShopgateOrder $oShopgateOrder)
    {
        $oInvoiceAddress = $oShopgateOrder->getInvoiceAddress();
        $oDeliverAddress = $oShopgateOrder->getDeliveryAddress();
        $oInvoiceAddress = $oInvoiceAddress->utf8Decode();
        $oDeliverAddress = $oDeliverAddress->utf8Decode();
        
        // check if the guest customer has been created already for the order to be imported (can happe on partial order imports)
        if (!empty($this->shopgateOrderData['orderID'])) {
            // check for existing customer
            $sql         = "
                SELECT userID
                FROM s_order
                WHERE id = '{$this->shopgateOrderData['orderID']}'
            ";
            $orderUserId = Shopware()->Db()->fetchOne($sql);
            if (empty($orderUserId)) {
                // should not happen by default but somehow there is no order for the given order id availably (possibly deleted order but left the shopgate order data alive)
                // -> reset shopware order id to zero as "not set" in that case
                Shopware()->Db()->update(
                    "s_shopgate_orders", array('orderID' => 0),
                    array("shopgate_order_number = '{$oShopgateOrder->getOrderNumber()}'")
                )
                ;
            } else {
                // no savings required, since there is already a shopware order with a valid user available
                // -> simply return the userID
                return $orderUserId;
            }
        }
        
        $importCustomer = array(
            'email'                 => $oShopgateOrder->getMail(),
            'accountmode'           => 1,
            'newsletter'            => 0,
            'phone'                 => ($oShopgateOrder->getPhone() ? $oShopgateOrder->getPhone() : ""),
            
            'billing_company'       => ($oInvoiceAddress->getCompany() ? $oInvoiceAddress->getCompany() : ""),
            'billing_salutation'    => ($oInvoiceAddress->getGender() == ShopgateAddress::MALE ? self::MALE
                : self::FEMALE),
            'billing_firstname'     => $oInvoiceAddress->getFirstName(),
            'billing_lastname'      => $oInvoiceAddress->getLastName(),
            'billing_street'        => ($oInvoiceAddress->getStreetName1() . ($oInvoiceAddress->getStreet2()
                    ? " ({$oInvoiceAddress->getStreet2()})" : "")),
            'billing_streetnumber'  => $oInvoiceAddress->getStreetNumber1(),
            'billing_zipcode'       => $oInvoiceAddress->getZipcode(),
            'billing_city'          => $oInvoiceAddress->getCity(),
            'billing_countryID'     => $this->getCountryByIso($oInvoiceAddress->getCountry()),
            // 			'billing_text6' 		=> 'generated by Shopgate',
            
            'shipping_company'      => ($oDeliverAddress->getCompany() ? $oDeliverAddress->getCompany() : ""),
            'shipping_salutation'   => ($oDeliverAddress->getGender() == ShopgateAddress::MALE ? self::MALE
                : self::FEMALE),
            'shipping_firstname'    => $oDeliverAddress->getFirstName(),
            'shipping_lastname'     => $oDeliverAddress->getLastName(),
            'shipping_street'       => ($oDeliverAddress->getStreetName1() . ($oDeliverAddress->getStreet2()
                    ? " ({$oDeliverAddress->getStreet2()})" : "")),
            'shipping_streetnumber' => $oDeliverAddress->getStreetNumber1(),
            'shipping_zipcode'      => $oDeliverAddress->getZipcode(),
            'shipping_city'         => $oDeliverAddress->getCity(),
            'shipping_countryID'    => $this->getCountryByIso($oDeliverAddress->getCountry()),
            // 			'shipping_text6' 		=> 'generated by Shopagate'
        );
        
        $importedUser = Shopware()->Api()->Import()->sCustomer($importCustomer);
        
        return $importedUser['userID'];
    }
    
    protected function insertOrderPaymentCosts($oOrder, ShopgateOrder $oShopgateOrder)
    {
        if (floatval($oShopgateOrder->getAmountShopPayment())) {
            $aItem                 = array();
            $aItem['articleID']    = 0;
            $aItem['ordernumber']  = "sw-payment-absolute";
            $aItem['priceNumeric'] = $oShopgateOrder->getAmountShopPayment();
            $aItem['quantity']     = 1;
            $aItem['articlename']  = utf8_decode("Zuschlag für Zahlungsart");
            $aItem['tax_rate']     = "19";
            $aItem['taxID']        = 0;
            
            $oOrder->sBasketData["content"][] = $aItem;
            
        }
        
        return $oOrder;
    }
    
    protected function insertOrderShippingCosts($aOrder, ShopgateOrder $oShopgateOrder)
    {
        
        if (floatval($oShopgateOrder->getAmountShipping())) {
            $taxID = 0;
            $tax   = 0;
            
            list($taxID, $tax) = $this->getOrderTax($oShopgateOrder);
            
            $shipping     = $oShopgateOrder->getAmountShipping();
            $shipping_net = $oShopgateOrder->getAmountShipping() / (1 + ($tax / 100));
            
            // round shipping amount to two digits to show tax at invoice pdf
            $shipping     = round($shipping, 2);
            $shipping_net = round($shipping_net, 2);
            
            $aOrder['invoice_shipping']     = $shipping;
            $aOrder['invoice_shipping_net'] = $shipping_net;
        }
        
        return $aOrder;
    }
    
    /**
     *
     * @param unknown_type  $oOrder
     * @param ShopgateOrder $oShopgateOrder
     */
    protected function insertOrderPayment($oOrder, ShopgateOrder $oShopgateOrder)
    {
        $aPayment = $this->getShopPaymentMethod($oShopgateOrder->getPaymentMethod());
        
        $oOrder->sUserData["additional"]["payment"] = $aPayment;
        
        return $oOrder;
    }
    
    protected function setOrderPayment(array $aShopwareOrder, ShopgateOrder $oShopgateOrder)
    {
        $internalComment = "";
        
        switch ($oShopgateOrder->getPaymentMethod()) {
// 			case ShopgateOrder::SHOPGATE:
// 				break;
            case ShopgateOrder::DEBIT:
                $this->setOrderPaymentUserDebit($aShopwareOrder, $oShopgateOrder);
                break;
// 			case ShopgateOrder::PREPAY:
// 				break;
// 			case ShopgateOrder::COD:
// 				break;
// 			case ShopgateOrder::INVOICE:
// 				break;
            case ShopgateOrder::PAYPAL:
                if ($this->paymentPaypalSupported) {
                    $this->setOrderPaymentPaypal($aShopwareOrder, $oShopgateOrder);
                }
                break;
// 			default:
// 				break;
        }
        
        foreach ($oShopgateOrder->getPaymentInfos() as $key => $value) {
            $internalComment .= "{$key} => {$value}\n";
        }
        
        $internalComment .= "\n";
        
        if ($oShopgateOrder->getPaymentTransactionNumber()) {
            $internalComment .= "Transaktions ID: {$oShopgateOrder->getPaymentTransactionNumber()}\n\n{$internalComment}";
        }
        
        $aShopwareOrder['internalcomment'] = $internalComment;
        Shopware()->Db()->update(
            "s_order", array("internalcomment" => $aShopwareOrder['internalcomment']), "id = {$aShopwareOrder['id']}"
        )
        ;
        
        return $aShopwareOrder;
    }
    
    protected function setOrderPaymentUserDebit(array $aShopwareOrder, ShopgateOrder $oShopgateOrder)
    {
        $aPaymentInfos = $oShopgateOrder->getPaymentInfos();
        
        $aUserDebit               = array();
        $aUserDebit['userID']     = $aShopwareOrder['userID'];
        $aUserDebit['account']    = $aPaymentInfos["bank_account_number"];
        $aUserDebit['bankholder'] = $aPaymentInfos["bank_account_holder"];
        $aUserDebit['bankcode']   = $aPaymentInfos["bank_code"];
        $aUserDebit['bankname']   = $aPaymentInfos["bank_name"];
        
        Shopware()->Db()->insert("s_user_debit", array($aUserDebit));
        
        // TODO IBAN AND BIC
        // CURRENTLY NOT AVAILABLE
        
        return $aShopwareOrder;
    }
    
    protected function setOrderPaymentPaypal(array $aOrder, ShopgateOrder $oShopgateOrder)
    {
        $aPaymentInfos = $oShopgateOrder->getPaymentInfos();
        
        $sql     =
            "SELECT transactionId FROM `paypal_orders` WHERE transactionId = '{$aPaymentInfos["transaction_id"]}'";
        $oObject = Shopware()->Db()->fetchOne($sql);
        
        $aPaypalInfos = array(
            "suid"          => $aOrder["user"]["customernumber"] ? $aOrder["user"]["customernumber"] : "",
            "payerId"       => $aPaymentInfos["payer_id"] ? $aPaymentInfos["payer_id"] : 0,
            "transactionId" => $aPaymentInfos["transaction_id"],
            "paymentStatus" => $oShopgateOrder->getIsPaid() ? "Complete" : "Pending",
            "authorization" => !$oShopgateOrder->getIsPaid(),
            "booked"        => $oShopgateOrder->getIsPaid(),
            "price"         => $oShopgateOrder->getAmountComplete(),
            "refunded"      => 0,
            "currency"      => "EUR", // TODO
            "stransid"      => "",
            //			"ipn",
            //			"dateTime"
        );
        
        Shopware()->Db()->query(
            "UPDATE `s_order` SET `transactionID` = '" . ($aOrder["transactionID"] . "#" . $aPaymentInfos["invnum"])
            . "' WHERE `id` = '{$aOrder["id"]}'"
        )
        ;
        
        if ($oObject) {
            Shopware()->Db()->update(
                "paypal_orders", $aPaypalInfos, array("transactionId = '{$aPaymentInfos["transaction_id"]}'")
            )
            ;
        } else {
            Shopware()->Db()->insert("paypal_orders", $aPaypalInfos);
        }
    }
    
    /**
     *
     * @param Order         $oOrder
     * @param ShopgateOrder $oShopgateOrder
     */
    protected function insertPlentyOrderData(Order $oOrder, ShopgateOrder $oShopgateOrder)
    {
        // Check if there is a plenty table
        $plentyTableName = 'plenty_order';
        if ($this->tableExists($plentyTableName)) {
            // Check if the order id has already been inserted to the plenty table
            $sql = "SELECT * FROM `{$plentyTableName}` WHERE `shopwareId` = " . $oOrder->getId();
            $row = Shopware()->Db()->fetchRow($sql);
            
            // Insert the order id to the plenty table since it does not exist, yet
            if (empty($row)) {
                $sql = "INSERT INTO `{$plentyTableName}` SET `shopwareId` = " . $oOrder->getId();
                Shopware()->Db()->query($sql);
            }
        }
        
        return $oOrder;
    }
    
    /**
     *
     * @param Order         $oOrder
     * @param ShopgateOrder $oShopgateOrder
     *
     * @return array $aShopwareOrder
     */
    protected function insertShopgateOrderData(array $aShopwareOrder, ShopgateOrder $oShopgateOrder)
    {
        if (empty($this->shopgateOrderData)) {
            $this->shopgateOrderData = array(
                "orderID"                          => 0,
                "shopgate_order_number"            => $oShopgateOrder->getOrderNumber(),
                "is_sent_to_shopgate"              => 0,
                "is_shipping_blocked"              => $oShopgateOrder->getIsShippingBlocked(),
                "is_cancellation_sent_to_shopgate" => 0,
                "reported_cancellations"           => "",
                "received_data"                    => serialize($oShopgateOrder),
                "order_import_complete"            => '0',
            );
            
            Shopware()->Db()->insert("s_shopgate_orders", $this->shopgateOrderData);
            $this->shopgateOrderData['id'] = Shopware()->Db()->lastInsertId("s_shopgate_orders");
        }
        
        return $aShopwareOrder;
    }
    
    /**
     *
     * @param int $oShopgateOrder
     */
    protected function markShopgateOrderImportComplete($shopgateOrderNumber)
    {
        Shopware()->Db()->update(
            "s_shopgate_orders", array('order_import_complete' => '1'),
            "shopgate_order_number = '{$shopgateOrderNumber}'"
        )
        ;
    }
    
    protected function insertOrder($oOrder, ShopgateOrder $oShopgateOrder)
    {
        
        $amountNet = 0;
        $maxTax    = 0;
        foreach ($oShopgateOrder->getItems() as $item) {
            $tax = (float)$item->getTaxPercent();
            if ($tax > $maxTax) {
                $maxTax = $tax;
            }
            
            $price    = $item->getUnitAmount();
            $quantity = $item->getQuantity();
            $infos    = $this->jsonDecode($item->getInternalOrderInfo(), true);
            
            if ($infos["purchasesteps"]) {
                $quantity = $quantity * $infos["purchasesteps"];
                $price    = $price / $infos["purchasesteps"];
            }
            
            $amountNet += $price * $quantity;
        }
        $shippingNet = $oShopgateOrder->getAmountShipping() / (1 + ($maxTax / 100));
        $amountNet += $shippingNet;
        
        $oOrder->sAmount                  = $oShopgateOrder->getAmountComplete();
        $oOrder->sAmountWithTax           = $oShopgateOrder->getAmountComplete();
        $oOrder->sAmountNet               = $amountNet;
        $oOrder->sShippingcosts           = $oShopgateOrder->getAmountShipping();
        $oOrder->sShippingcostsNumeric    = $oShopgateOrder->getAmountShipping();
        $oOrder->sShippingcostsNumericNet = $shippingNet;
        $oOrder->bookingId                = $oShopgateOrder->getOrderNumber();
        
        $aBasket                          = array();
        $aBasket["Amount"]                = $oShopgateOrder->getAmountComplete();
        $aBasket["AmountNet"]             = $amountNet;
        $aBasket["Quantity"]              = 1;
        $aBasket["AmountNumeric"]         = $oShopgateOrder->getAmountComplete();
        $aBasket["AmountNetNumeric"]      = $amountNet;
        $aBasket["AmountWithTax"]         = 0;
        $aBasket["AmountWithTaxNumeric"]  = 0;
        $aBasket["sShippingcostsWithTax"] = $oShopgateOrder->getAmountShipping();
        $aBasket["sShippingcostsNet"]     = $shippingNet;
        $aBasket["sShippingcostsTax"]     = $maxTax;
// 		$aBasket["sTaxRates"] = array( "19" => "6.38" );
        $aBasket["sShippingcosts"] = 0;
// 		$aBasket["sAmount"] = 39.95;
// 		$aBasket["sAmountTax"] = 6.38;
        
        $oOrder->sBasketData = $aBasket;
        
        $oOrder->dispatchId = $this->config->getShippingService();
        
        return $oOrder;
    }
    
    protected function prepareShopgateOrder(ShopgateOrder $oShopgateOrder)
    {
        if (!$oShopgateOrder->getPhone()) {
            $oShopgateOrder->setPhone("");
        }
        if (!$oShopgateOrder->getInvoiceAddress()->getPhone()) {
            $oShopgateOrder->getInvoiceAddress()->setPhone($oShopgateOrder->getPhone());
        }
        if (!$oShopgateOrder->getDeliveryAddress()->getPhone()) {
            $oShopgateOrder->getDeliveryAddress()->setPhone($oShopgateOrder->getPhone());
        }
        
        return $oShopgateOrder;
    }
    
    /**
     *
     * @param unknown_type  $oOrder
     * @param ShopgateOrder $oShopgateOrder
     */
    protected function insertOrderCustomer($oOrder, ShopgateOrder $oShopgateOrder)
    {
        $extCustomerNumber = $oShopgateOrder->getExternalCustomerNumber();
        $oInvoice          = $oShopgateOrder->getInvoiceAddress();
        
        if ($extCustomerNumber) {
            $sql    = "
                SELECT userID
                FROM s_user_billingaddress
                WHERE customernumber = '" . $extCustomerNumber . "'
            ";
            $userID = Shopware()->Db()->fetchOne($sql);
        } else {
            $userID = $this->createUser($oShopgateOrder);
        }
        
        $payment = $this->getShopPaymentMethod($oShopgateOrder->getPaymentMethod());
        
        $aUser                  = array();
        $aUser["id"]            = $userID;
        $aUser["email"]         = $oShopgateOrder->getMail();
        $aUser["active"]        = 0;
        $aUser["accountmode"]   = 1;
        $aUser["newsletter"]    = 0;
        $aUser["customergroup"] = "EK";
        $aUser["paymentpreset"] = 0;
        $aUser["language"]      = "";
        $aUser["subshopID"]     = $this->subShop['id'];
        $aUser["paymentID"]     = $payment["id"];
        
        $oOrder->sUserData["additional"]["user"] = $aUser;
        
        return $oOrder;
    }
    
    protected function getShopCustomer($userId)
    {
        $user = $this->export->sCustomers(array("userIDs" => array($userId)));
        
        return array_shift($user);
    }
    
    /**
     *
     * @param unknown_type  $oOrder
     * @param ShopgateOrder $oShopgateOrder
     */
    protected function insertOrderDeliveryAddress($oOrder, ShopgateOrder $oShopgateOrder)
    {
        return $this->insertOrderAddress($oOrder, $oShopgateOrder->getDeliveryAddress(), "shippingaddress");
    }
    
    /**
     *
     * @param unknown_type  $oOrder
     * @param ShopgateOrder $oShopgateOrder
     */
    protected function insertOrderInvoiceAddress($oOrder, ShopgateOrder $oShopgateOrder)
    {
        $oOrder = $this->insertOrderAddress($oOrder, $oShopgateOrder->getDeliveryAddress(), "billingaddress");
        
        $oOrder->sUserData["billingaddress"]["customernumber"] =
            $oShopgateOrder->getExternalCustomerNumber() ? $oShopgateOrder->getExternalCustomerNumber() : "";
        $oOrder->sUserData["billingaddress"]["phone"]          =
            $oShopgateOrder->getPhone() ? $oShopgateOrder->getPhone() : "";
        
        
        return $oOrder;
    }
    
    protected function insertOrderAddress($oOrder, ShopgateAddress $oOrderAddress, $type)
    {
        $oOrderAddress = $oOrderAddress->utf8Decode();
        $country       = $this->getCountryByIso($oOrderAddress->getCountry());
        
        $aAddress                 = array();
        $aAddress["userID"]       = $oOrder->sUserData["additional"]["user"]["id"];
        $aAddress["department"]   = "";
        $aAddress["company"]      = $oOrderAddress->getCompany() ? $oOrderAddress->getCompany() : "";
        $aAddress["salutation"]   = ($oOrderAddress->getGender() == ShopgateAddress::MALE) ? self::MALE : self::FEMALE;
        $aAddress["firstname"]    = $oOrderAddress->getFirstName();
        $aAddress["lastname"]     = $oOrderAddress->getLastName();
        $aAddress["street"]       =
            $oOrderAddress->getStreetName1() . ($oOrderAddress->getStreet2() ? " ({$oOrderAddress->getStreet2()})"
                : "");
        $aAddress["streetnumber"] = $oOrderAddress->getStreetNumber1();
        $aAddress["zipcode"]      = $oOrderAddress->getZipcode();
        $aAddress["city"]         = $oOrderAddress->getCity();
        $aAddress["countryID"]    = $country;
        $aAddress["phone"]        = "";
        $aAddress["fax"]          = "";
        $aAddress["ustid"]        = "";
        $aAddress["birthday"]     = "";
        $aAddress["text1"]        = "";
        $aAddress["text2"]        = "";
        $aAddress["text3"]        = "";
        $aAddress["text4"]        = "";
        $aAddress["text5"]        = "";
        $aAddress["text6"]        = "";
        
        if ($type == "shippingaddress") {
            $isEqualBilling = true;
            foreach ($aAddress as $key => $value) {
                if ($oOrder->sUserData["billingaddress"][$key] != $value) {
                    $isEqualBilling = false;
                    break;
                }
            }
            
            $aAddress["eqalBilling"] = $isEqualBilling ? "1" : "0";
        }
        
        $oOrder->sUserData[$type] = $aAddress;
        
        return $oOrder;
    }
    
    protected function insertOrderCountry($oOrder, ShopgateOrder $oShopgateOrder)
    {
        $aCountry                          = array();
        $aCountry["id"]                    = "2";
        $aCountry["countryname"]           = "Deutschland";
        $aCountry["countryiso"]            = "DE";
        $aCountry["areaID"]                = "1";
        $aCountry["countryen"]             = "GERMANY";
        $aCountry["position"]              = "1";
        $aCountry["notice"]                = "";
        $aCountry["shippingfree"]          = "0";
        $aCountry["taxfree"]               = "0";
        $aCountry["taxfree_ustid"]         = "0";
        $aCountry["taxfree_ustid_checked"] = "0";
        $aCountry["active"]                = "1";
        $aCountry["iso3"]                  = "DEU";
        
        $oOrder->sUserData["additional"]["country"] = $aCountry;
        
        return $oOrder;
    }
    
    protected function insertOrderCountryShipping($oOrder, ShopgateOrder $oShopgateOrder)
    {
        $aCountry                                  = array();
        $aCountry["id"]                            = "2";
        $aCountry["countryname"]                   = "Deutschland";
        $aCountry["countryiso"]                    = "DE";
        $aCountry["areaID"]                        = "1";
        $aCountry["countryen"]                     = "GERMANY";
        $aCountry["position"]                      = "1";
        $aCountry["notice"]                        = "";
        $aCountry["shippingfree"]                  = "0";
        $aCountry["taxfree"]                       = "0";
        $aCountry["taxfree_ustid"]                 = "0";
        $aCountry["taxfree_ustid_checked"]         = "0";
        $aCountry["active"]                        = "1";
        $aCountry["iso3"]                          = "DEU";
        $aCountry["display_state_in_registration"] = "0";
        $aCountry["force_state_in_registration"]   = "0";
        $aCountry["countryarea"]                   = "deutschland";
        
        $oOrder->sUserData["additional"]["countryShipping"] = $aCountry;
        
        return $oOrder;
    }
    
    protected function insertOrderItems($oOrder, ShopgateOrder $oShopgateOrder)
    {
        $aItems = array();
        
        foreach ($oShopgateOrder->getItems() as $oItem) {
            /* @val $oItem ShopgateOrderItem */
            
            $oItem = $oItem->utf8Decode("ISO-8859-15");
            
            $info = $this->jsonDecode($oItem->getInternalOrderInfo(), true);
            
            $priceNet = $oItem->getUnitAmount();
            $price    = $oItem->getUnitAmountWithTax();
            $quantity = $oItem->getQuantity();
            
            if ($info["purchasesteps"]) {
                $quantity = $quantity * $info["purchasesteps"];
                $price    = $price / $info["purchasesteps"];
                $priceNet = $priceNet / $info["purchasesteps"];
            }
            
            $images = Shopware()->Modules()->Articles()->sGetArticlePictures(
                $info["articleID"], true, 0, $oItem->getItemNumber()
            )
            ;
            
            $aItem              = array();
            $aItem['articleID'] = $info["articleID"] ? $info["articleID"] : 0;
            
            $aItem['ordernumber'] = $oItem->getItemNumber();
            
            $aItem['netprice']     = $this->formatPriceNumber($priceNet);
            $aItem['price']        = $this->formatPriceNumber($price);
            $aItem['priceNumeric'] = $price;
            
            $aItem['amountnet'] = $this->formatPriceNumber($priceNet * $quantity);
            $aItem['amount']    = $this->formatPriceNumber($price * $quantity);
            
            $aItem['quantity']    = $quantity;
            $aItem['articlename'] =
                $this->_buildOrderArticleName($oItem->getName(), $oItem->getAttributes(), $oItem->getOptions());
            $aItem['name']        = $aItem['articlename'];
// 			$aItem['status']		= 0;
            $aItem['tax_rate'] = $oItem->getTaxPercent();
            $aItem['taxID']    = !empty($info["tax_id"]) ? $info["tax_id"] : 0;
            $aItem['image']    = $images;
            
            $aItems[] = $aItem;
        }
        
        $oOrder->sBasketData["content"] = $aItems;
        
        return $oOrder;
    }
    
    protected function insertOrderExternalCoupons($oOrder, ShopgateOrder $oShopgateOrder)
    {
        foreach ($oShopgateOrder->getExternalCoupons() as $coupon) {
            $info = $this->jsonDecode($coupon->getInternalInfo(), true);
            
            $aItem              = array();
            $aItem['articleID'] = $info["voucher_id"];
            
            $aItem['ordernumber'] = $info["order_code"];
            
            $tax      = $info["tax_percent"];
            $price    = abs($coupon->getAmount()) * -1; // be sure that coupon amount is negative!
            $priceNet = $price / (1 + ($tax / 100));
            
            $aItem['netprice']     = $this->formatPriceNumber($priceNet);
            $aItem['price']        = $this->formatPriceNumber($price);
            $aItem['priceNumeric'] = $price;
            
            $aItem['amountnet'] = $this->formatPriceNumber($priceNet);
            $aItem['amount']    = $this->formatPriceNumber($price);
            
            $aItem['quantity']    = 1;
            $aItem['articlename'] = "Gutschein";
            $aItem['status']      = 0;
            $aItem['modus']       = 2;
            $aItem['tax_rate']    = $tax;
            $aItem['taxID']       = 0;
            
            $oOrder->sBasketData["content"][] = $aItem;
            
            if ($info["code_id"]) {
                Shopware()->Db()->update(
                    "s_emarketing_voucher_codes", array('userID' => $oOrder->sUserData["additional"]["user"]["id"]),
                    array("id = '{$info["code_id"]}'")
                )
                ;
            }
        }
        
        return $oOrder;
    }
    
    protected function getShopPaymentMethod($sPaymentName)
    {
        switch ($sPaymentName) {
            case ShopgateOrder::SHOPGATE:
                $sPaymentName = "shopgate";
                break;
            case ShopgateOrder::DEBIT:
                $sPaymentName = "debit";
                break;
            case ShopgateOrder::PREPAY:
                $sPaymentName = "prepayment";
                break;
            case ShopgateOrder::COD:
                $sPaymentName = "cash";
                break;
            case ShopgateOrder::INVOICE:
                $sPaymentName = "invoice";
                break;
            case ShopgateOrder::PAYPAL:
                if ($this->paymentPaypalSupported) {
                    $sPaymentName = self::PAYMENT_NAME_PAYPAL;
                } else {
                    $sPaymentName = "mobile_payment";
                }
                break;
            default:
                $sPaymentName = "mobile_payment";
                break;
        }
        
        $sql = "SELECT * FROM `s_core_paymentmeans` WHERE name = '{$sPaymentName}'";
        
        $payment = Shopware()->Db()->fetchRow($sql);
        
        return $payment;
    }
    
    protected function getOrderItemName(ShopgateOrderItem $item)
    {
        $itemName = $item->getName();
        
        $infos     = $this->jsonDecode($item->getInternalOrderInfo(), true);
        $articleId = $infos["articleID"] ? $infos["articleID"] : 0;
        
        if ($articleId) {
            $article = $this->getArticleById($articleId);
            
            if (!empty($article['sVariants'])) {
                foreach ($article['sVariants'] as $variant) {
                    if ($variant["ordernumber"] == $item->getItemNumber()) {
                        $itemName .= " - {$variant["additionaltext"]}";
                        break;
                    }
                }
            } else if ($this->checkIfConfig($article['articleID'])) {
                if ($article['sConfiguratorSettings']['type'] == 3) {
                    $itemName = $this->buildOrderArticleName($item->getName(), $item->getOptions());
                } else {
                    $sql = "
                        SELECT
                            agv.*,
                            agp.price
                        FROM s_articles_groups_value agv
                        JOIN s_articles_groups_prices agp
                          ON agv.valueID = agp.valueID
                        WHERE agv.articleID = '" . $article['articleID'] . "'
                          AND agv.ordernumber = '{$item->getItemNumber()}'
                          AND agv.active 		= 1
                          AND agp.groupkey 	= '" . $this->subShop['defaultcustomergroup'] . "'
                        ";
                    
                    $result = Shopware()->Db()->query($sql);
                    
                    $strings = array();
                    while ($row = $result->fetch()) {
                        for ($i = 1; $i <= 10; $i++) {
                            if ($row["attr{$i}"]) {
                                $strings[] =
                                    "{$article['sConfigurator'][$i-1]['groupname']}: {$this->getOptionName($row["attr{$i}"])}";
                            }
                        }
                    }
                    
                    $itemName .= " - " . implode(" / ", $strings);
                }
            }
        }
        
        return $itemName;
    }
    
    protected function updateArticleStock(ShopgateOrderItem $item)
    {
        // fetch required product data from database
        $sql       = "
            SELECT
                ad.articleID,
                a.laststock,
                ad.instock
            FROM `s_articles` a
            JOIN `s_articles_details` ad ON ( a.id = ad.articleID )
            WHERE ad.ordernumber = '{$item->getItemNumber()}'";
        $stockData = Shopware()->Db()->fetchRow($sql);
        
        // Article doesnt exist any more
        if (!$stockData) {
            return;
        }
        
        $infos    = $this->jsonDecode($item->getInternalOrderInfo(), true);
        $quantity = $item->getQuantity();
        if ($infos["purchasesteps"]) {
            $quantity = $quantity * $infos["purchasesteps"];
        }
        
        // calculate the new stock
        $stock = $stockData["instock"] - $quantity;
        
        // data to update on article details
        $updateData = array();
        
        // set the new stock
        $updateData["instock"] = $stock;
        
        // if out of stock and should be deactivate - deactivate article detail
        if ($stock <= 0 && $this->sSYSTEM->sCONFIG['sDEACTIVATENOINSTOCK'] == 1
            && $stockData["laststock"] == 1
        ) {
            
            $this->log("Deactivate Article-Details (id: {$item->getItemNumber()})", ShopgateLogger::LOGTYPE_ACCESS);
            $updateData["active"] = "0";
        }
        
        // execute the update query
        Shopware()->Db()->update(
            "s_articles_details",
            $updateData,
            "ordernumber = '{$item->getItemNumber()}'"
        )
        ;
        
        // count active article details
        $countActive = Shopware()->Db()->fetchOne(
            "SELECT COUNT(id) FROM `s_articles_details`
                WHERE active = 1 AND articleID = '{$stockData["articleID"]}'"
        )
        ;
        
        // if there are no active details then deactivate the article
        if ($countActive == 0) {
            $this->log("Deactivate Article (id: {$stockData["articleID"]})", ShopgateLogger::LOGTYPE_ACCESS);
            Shopware()->Db()->update(
                "s_articles",
                array("active" => "0"),
                "id = {$stockData["articleID"]}"
            )
            ;
        }
    }
    
    protected function getPaymentId($sPaymentName)
    {
        $sql = "SELECT id FROM `s_core_paymentmeans` WHERE name = '{$sPaymentName}'";
        $iId = Shopware()->Db()->fetchOne($sql);
        
        return $iId;
    }
    
    protected function setOrderClear(array $aShopwareOrder, ShopgateOrder $oShopgateOrder)
    {
        $iStatus = 17; // Offen
        $sDate   = null;
        
        if ($oShopgateOrder->getIsPaid()) {
            $iStatus = 12; // Komplett bezahlt
            $sDate   = $oShopgateOrder->getPaymentTime("Y-m-d H:i:s");
        }
        
        $aShopwareOrder["cleared"]     = $iStatus;
        $aShopwareOrder['cleareddate'] = $sDate;
        
        Shopware()->Db()->update(
            "s_order",
            array(
                "cleared"     => $aShopwareOrder["cleared"],
                "cleareddate" => $aShopwareOrder['cleareddate']
            ),
            "id = {$aShopwareOrder['id']}"
        )
        ;
        
        return $aShopwareOrder;
    }
    
    protected function setOrderStatus(array $aShopwareOrder, ShopgateOrder $oShopgateOrder)
    {
        $iShippingBlockedStatus = 8; // Klaerung notwendig
        
        // Do this only on following conditions met:
        if ($aShopwareOrder['status'] == 0
            // it is called while updating an order, the order has possibly not been imported correctly or the status hasn't been changed
            || $oShopgateOrder->getUpdateShipping()
        ) { // called while updating an order
            $iStatus = $iShippingBlockedStatus;
            
            if (!$oShopgateOrder->getIsShippingBlocked()) {
                $iStatus = $this->config->getShippingNotBlockedStatus();
            }
            
            $aShopwareOrder["status"] = $iStatus;
            
            Shopware()->Db()->update(
                "s_order",
                array(
                    "status" => $aShopwareOrder["status"],
                ),
                "id = {$aShopwareOrder['id']}"
            )
            ;
        }
        
        return $aShopwareOrder;
    }
    
    /**
     *
     * use list($taxID, $tax) = $this->getOrderTax($order) to fetch the to variables
     *
     * @param ShopgateOrder $order
     *
     * @return array(taxID, $tax)
     */
    protected function getOrderTax(ShopgateOrder $order)
    {
        $taxId = 0;
        $tax   = 0;
        
        foreach ($order->getItems() as $item) {
            /* @var $item ShopgateOrderItem */
            
            if ($item->getTaxPercent() > $tax) {
                $infos = $this->jsonDecode($item->getInternalOrderInfo(), true);
                $tax   = $item->getTaxPercent();
                $taxId = $infos["tax_id"];
            }
        }
        
        return array($taxId, $tax);
    }
    
    ############################################################################
    ## Settings
    ############################################################################
    
    protected function getExportSettingsLoaders()
    {
        return array(
            "settingsExportProductTaxClasses", /** $this->settingsExportProductTaxClasses */
            "settingsExportCustomerTaxClasses", /** $this->settingsExportCustomerTaxClasses */
            "settingsExportTaxRates", /** $this->settingsExportTaxRates */
            "settingsExportTaxRules",
            /** $this->settingsExportTaxRules */
        );
    }
    
    public function getSettings()
    {
        $settings = array(
            "tax" => array(
                "product_tax_classes"  => array(),
                "customer_tax_classes" => array(),
                "tax_rates"            => array(),
                "tax_rules"            => array(),
            )
        );
        
        $settings = $this->executeLoaders($this->getExportSettingsLoaders(), $settings);
        
        return $settings;
    }
    
    protected function settingsExportProductTaxClasses(array $settings)
    {
        $classes = array();
        
        $qry = Shopware()->Db()->query("SELECT * FROM `s_core_tax`");
        
        foreach ($qry->fetchAll() as $tax) {
            /* @var $tax array(id, tax, description */
            
            $classes[] = array(
                "id"  => $tax["id"],
                "key" => "tax_{$tax["tax"]}",
            );
        }
        
        $settings["tax"]["product_tax_classes"] = $classes;
        
        return $settings;
    }
    
    protected function settingsExportCustomerTaxClasses(array $settings)
    {
        $groups = array();
        
        $groups[] = array(
            "id"         => "default",
            "key"        => "default",
            "is_default" => "1",
        );
        
        $settings["tax"]["customer_tax_classes"] = $groups;
        
        return $settings;
    }
    
    protected function settingsExportTaxRates(array $settings)
    {
        $rates = array();
        
        $qry = Shopware()->Db()->query("SELECT * FROM `s_core_tax`");
        
        foreach ($qry->fetchAll() as $tax) {
            /* @var $tax array(id, tax, description */
            
            $rates[] = array(
                "id"                 => $tax["id"],
                "key"                => "rate_{$tax["tax"]}",
                "display_name"       => $tax["description"],
                "tax_percent"        => $tax["tax"],
                "country"            => "",
                "state"              => "",
                "zipcode_type"       => "all",
                "zipcode_pattern"    => "",
                "zipcode_range_from" => "",
                "zipcode_range_to"   => "",
            );
        }
        
        $settings["tax"]["tax_rates"] = $rates;
        
        return $settings;
    }
    
    protected function settingsExportTaxRules(array $settings)
    {
        $rules = array();
        
        $qry = Shopware()->Db()->query("SELECT * FROM `s_core_tax`");
        
        foreach ($qry->fetchAll() as $tax) {
            /* @var $tax array(id, tax, description */
            
            $_rule = array(
                "id"                   => "",
                "key"                  => "rule_{$tax["tax"]}",
                "name"                 => "Rule {$tax["description"]}",
                "priority"             => 0,
                "product_tax_classes"  => array(array("id" => $tax["id"], "key" => "tax_{$tax["tax"]}")),
                "customer_tax_classes" => array(array("id" => "default", "key" => "default")),
                "tax_rates"            => array(array("id" => $tax["id"], "key" => "rate_{$tax["tax"]}")),
            );
            
            $rules[] = $_rule;
        }
        
        $settings["tax"]["tax_rules"] = $rules;
        
        return $settings;
    }
    
    /**
     * Checks if the given table exists
     *
     * @param string $tableName
     *
     * @throws ShopgateLibraryException
     * @return boolean
     */
    protected function tableExists($tableName)
    {
        $tableName = trim($tableName);
        if (empty($tableName)) {
            return false;
        }
        
        // Get all table names
        $result = Shopware()->Db()->fetchAll("SHOW TABLES");
        
        foreach ($result as $row) {
            $row = array_values($row);
            
            // Check for table name
            if ($row[0] == $tableName) {
                return true;
            }
        }
        
        // The requested table has not been found if execution reaches here
        return false;
    }
}
